/*
 * smi/appConf/ActionConfiguration.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.appConf;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.security.InvalidParameterException;

import ssonet.SSONETConstants;

/** This class saves all values which are neccessary for configuring a action.
* This are mainly settings for the security goals. Methods are provided to access the values.
* Furthermore there are methods to save a configuration into a stream, respectively read from there.
*/
final public class ActionConfiguration {
/** A short description of the security goal levels
    */
    private final static String names[]={
	"on no condition",
	    "on no condition, negotiable",
	    "if neccessary",
	    "dont care",
	    "if possible",
	    "unconditional, negotiable",
	    "unconditional"
    };

    /** contains the size (in bytes) which a configuration file needs if its saved in a stream
    * or is read from one
    */
    public final static int STREAM_SIZE=6;

    /** value for security goal confidentiality */
    private byte confidentiality;

    /** value for security goal anonymity */
    private byte anonymity;

    /** value for security goal accountability */
    private byte accountability;

    /** value for security goal integrity */
    private byte integrity;

    /** unique identifier for the action of this configuration
    */
    private short actionID;

    /** creates a action configuration objectes
    * @param actionID unique action number
    * @param confidentiality Level for confidentiality
    * @param anonymity Level for anonymity
    * @param integrity Level for Integrity
    * @param accountability Level for accountability
    * The value for the level to set ranges from 0 - 6. For a more precise description of the
    * number meanings see @see names
    */
    public ActionConfiguration(short actionID,
	int confidentiality,
	int anonymity,
	int integrity,
	int accountability) throws InvalidParameterException {
	// set values
	if(!setSecurityGoalLevel(SSONETConstants.CONFIDENTIALITY,(byte)confidentiality))
	    throw new InvalidParameterException();
	if(!setSecurityGoalLevel(SSONETConstants.ANONYMITY,(byte)anonymity))
	    throw new InvalidParameterException();
	if(!setSecurityGoalLevel(SSONETConstants.INTEGRITY,(byte)integrity))
	    throw new InvalidParameterException();
	if(!setSecurityGoalLevel(SSONETConstants.ACCOUNTABILITY_SIGN,(byte)accountability))
	    throw new InvalidParameterException();
	this.actionID=actionID;
    }

    /** Returns a short description of the set level for the corresponding security goal.
    * The security goal constants are defined in @see SSONETConstants
    * @return short description of the set level, see @see names
    */
    public String toString(int securityGoal) {
	switch (securityGoal) {
	case SSONETConstants.CONFIDENTIALITY:
	    return names[confidentiality];
	case SSONETConstants.ACCOUNTABILITY_SIGN:
	    return names[accountability];
	case SSONETConstants.INTEGRITY:
	    return names[integrity];
	case SSONETConstants.ANONYMITY:
	    return names[anonymity];
	}
	// invalid security goal
	return null;
    }

    /** returns a text, from which one can extract the actual settings
    * @return description of the actual settings
    */
    public String toString() {
	return
	    "Configuration for actionID: " + Integer.toString(actionID) +
	    " Confidentiality: " + toString(SSONETConstants.CONFIDENTIALITY) +
	    " Anonymity: " + toString(SSONETConstants.ANONYMITY) +
	    " Accountability: " + toString(SSONETConstants.ACCOUNTABILITY_SIGN) +
	    " Integrity: " + toString(SSONETConstants.INTEGRITY);
    }


    /** returns the level set for a security goal
    * @param securityGoal security goal the level should be determined for.
    * See @see SSONEConstants
    * @return level set
    * -1, if security goal not valid
    */
    public byte getSecurityGoalLevel(int securityGoal) {
	switch (securityGoal) {
	case SSONETConstants.CONFIDENTIALITY:
	    return confidentiality;
	case SSONETConstants.ACCOUNTABILITY_SIGN:
	    return accountability;
	case SSONETConstants.INTEGRITY:
	    return integrity;
	case SSONETConstants.ANONYMITY:
	    return anonymity;
	}
	return -1;
    }

    /** Sets the level for a security goal
    * @param securityGoal security goal the level should be set for.
    * See @see SSONEConstants
    * @param securityLevel securityLevel to set
    * @return true, if security goal set to securityLevel
    * false, if securityLevel or security goal are invalid
    */
    public boolean setSecurityGoalLevel(int securityGoal, byte securityLevel) {
	if((securityLevel < SSONETConstants.LEVEL_MIN) || (securityLevel > SSONETConstants.LEVEL_MAX)) return false;

	switch (securityGoal) {
	case SSONETConstants.CONFIDENTIALITY:
	    confidentiality = securityLevel;
	    break;
	case SSONETConstants.ACCOUNTABILITY_SIGN:
	    accountability = securityLevel;
	    break;
	case SSONETConstants.INTEGRITY:
	    integrity = securityLevel;
	    break;
	case SSONETConstants.ANONYMITY:
	    anonymity = securityLevel;
	    break;
	default:
	    return false;
	}
	return true;
    }

    /** Raises the level for a security goal.
    * 'on no condition' becomes 'if neccessary'; 'if neccessary' becomes 'dont care' etc.
    * @param securityGoal security goal which security level should be raised
    * @return true, if settings are changed successfully
    * false, if security goal is invalid or cant be raised anymore
    */
    public boolean raise(int securityGoal) {
	byte securityLevel = getSecurityGoalLevel(securityGoal);
	switch (securityLevel)
	{
	case SSONETConstants.ON_NO_CONDITION:
	case SSONETConstants.ALWAYS_ON_NO_CONDITION:
	    securityLevel = SSONETConstants.IF_NECCESSARY;
	    break;
	case SSONETConstants.IF_NECCESSARY:
	    securityLevel = SSONETConstants.DONT_CARE;
	    break;
	case SSONETConstants.DONT_CARE:
	    securityLevel = SSONETConstants.IF_POSSIBLE;
	    break;
	case SSONETConstants.IF_POSSIBLE:
	    securityLevel = SSONETConstants.UNCONDITIONAL;
	    break;
	default:
	    return false;
	}
	return setSecurityGoalLevel(securityGoal, securityLevel);
    }

    /** Lowers the level for a security goal.
    * 'if neccessary' becomes 'on no condition'; 'dont care' becomes 'if neccessary' etc.
    * @param securityGoal security goal which security level should be lowered
    * @return true, if settings are changed successfully
    * false, if security goal is invalid or cant be lowered anymore
    */
    public boolean lower(int securityGoal) {
	byte securityLevel = getSecurityGoalLevel(securityGoal);
	switch (securityLevel) {
	case SSONETConstants.IF_NECCESSARY:
	    securityLevel = SSONETConstants.ON_NO_CONDITION;
	    break;
	case SSONETConstants.DONT_CARE:
	    securityLevel = SSONETConstants.IF_NECCESSARY;
	    break;
	case SSONETConstants.IF_POSSIBLE:
	    securityLevel = SSONETConstants.DONT_CARE;
	    break;
	case SSONETConstants.UNCONDITIONAL:
	case SSONETConstants.ALWAYS_UNCONDITIONAL:
	    securityLevel = SSONETConstants.IF_POSSIBLE;
	    break;
	default:
	    return false;
	}
	return setSecurityGoalLevel(securityGoal, securityLevel);
    }

    /** returns the action ID
    * @return actionID
    */
    public short getID() {
	return actionID;
    }

    /** sets the action ID
    * @param actionID new action number
    */
    public void setID(short actionID) {
	this.actionID = actionID;
    }

    /** Outputs the set configuration to a stream
    * Regarding negotiableIsSecret the setting negotiable will be suppressed:
    * on no condition, negotiable' becomes on no condition and 'unconditional, negotiable becomes 'unconditional
    * The format is as follows:
    *	Byte 0,1: action number
    *	Byte 2	: value for confidentiality
    *	Byte 3	: value for anonymity
    *	Byte 4	: value for integrity
    *	Byte 5	: value for accountability
    * @param outputStream OutputStream
    * @param negotiableIsSecret determines, if negotiable is suppressed.  true - negotiable will be suppressed
    * @exception IOException if an error occured during writing
    */
    public void toStream(OutputStream outputStream, boolean negotiableIsSecret) throws IOException {
	try {
	    byte[] buff = new byte[6];
	    buff[0] = (byte)((actionID>>8)&0x000000FF);
	    buff[1] = (byte)(actionID&0x000000FF);
	    buff[2] = confidentiality;
	    buff[3] = anonymity;
	    buff[4] = integrity;
	    buff[5] = accountability;
	    if (negotiableIsSecret)
		// negotiable should not be seen by the partner
		for(int i = 2; i < 6; i++)
		switch(buff[i]) {
	case SSONETConstants.UNCONDITIONAL:
	    buff[i] = SSONETConstants.ALWAYS_UNCONDITIONAL;
	    break;
	case SSONETConstants.ON_NO_CONDITION:
	    buff[i] = SSONETConstants.ALWAYS_ON_NO_CONDITION;
	    break;
	    }
	    outputStream.write(buff);
	} catch(Exception e) {
	    throw new IOException();
	}
    }

    /** reads a action configuration from a stream
    * @param inputStream InputStream
    * @return read configuration
    * null, if an error occured during reading
    */
    public static ActionConfiguration fromStream(InputStream inputStream) {
	try {
	    byte[] buffer = new byte[6];
	    inputStream.read(buffer);
	    short actionID = buffer[0];
	    actionID <<= 8;
	    actionID &= 0x0000FF00;
	    actionID |= (buffer[1] & 0x000000FF);
	    return new ActionConfiguration(actionID,
		buffer[2],
		buffer[3],
		buffer[4],
		buffer[5]);
	} catch(Exception e) {
	    return null;
	}
    }
}
