/*
 * smi/mechanisms/AbstractConfidentialityMechanism.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import xjava.security.Cipher;
import java.security.Key;
import xjava.security.KeyGenerator;
import java.security.SecureRandom;
import java.util.StringTokenizer;

import javax.swing.JFrame;

import smi.baseConf.RatedString;
import smi.io.IOStream;
import smi.mechanisms.MechanismConfiguration;

import ssonet.KeyExchange;
import ssonet.SSONETConnection;
import ssonet.SSONETConstants;

import sun.security.x509.X509Cert;

public class AbstractConfidentialityMechanism implements IOStream, Mechanism {
    protected IOStream ioStream;
    protected Cipher cipherIn, cipherOut;
    protected static String PROVIDERLIST = "Providerlist";
    protected static String KEY_MODE_PREFERENCELIST = "Mode.Preferencelist";
    protected static String KEY_KEYLENGTH = "KeyLength";
    protected static String KEY_ROUNDS = "Rounds";
    protected static String KEY_VARIANT = "Variant";
    protected String mechanismName;
    String defaultProvider[] = {"Cryptix"};
    public String[] getModes() {
	String[] modes = {"CBC","CFB","ECB","OFB","PCBC"};
	return modes;
    }

    public int available() {
	return ioStream.available();
    }

    public void close() {
	ioStream.close();
    }

    /**
    * Encrypt buffer and return the encrypted data
    * @param buffer  date to crypt
    * @return  encrypted buffer data
    */
    public byte[] testEncrypt (byte[] buffer) {
        return cipherOut.crypt(buffer);
    }

    /**
    * Decrypt buffer and return the decrypted data
    * @param buffer  date to crypt
    * @return  decrypted buffer data
    */
    public byte[] testDecrypt (byte[] buffer) {
        return cipherIn.crypt(buffer);
    }

    public Object readObject() throws Exception {
	try {
	    byte[] buffer = (byte[])ioStream.readObject();
	    buffer = cipherIn.crypt(buffer);
	    return (new ObjectInputStream(new ByteArrayInputStream(buffer))).readObject();
	} catch (Exception e) {
	    System.out.println("Error in AbstractConfidentialityMechanism.readObject");
	    e.printStackTrace();
	    throw e;
	}
    }

    public void writeObject(Object object) throws Exception {
	try {
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(object);
	    objectOutputStream.flush();
	    byte[] buffer = byteArrayOutputStream.toByteArray();
	    buffer = cipherOut.crypt(buffer);
	    ioStream.writeObject(buffer);
	    objectOutputStream.close();
	    byteArrayOutputStream.close();
	} catch (Exception e) {
	    System.out.println("Error in AbstractConfidentialityMechanism.writeObject");
	    e.printStackTrace();
	    throw e;
	}
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {
	    this.ioStream = ioStream;
	    String[] modePreferenceList = (String[])ownMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String mode = modePreferenceList[0];
	    System.out.println("Using mode: " + mode);

	    StringTokenizer stringTokenizer = new StringTokenizer(getClass().getName(),".");
	    for (int i = 0; i<4; i++) {
		mechanismName = stringTokenizer.nextToken();
	    }

	    System.out.println("generating " + mechanismName + " Key.");
	    SecureRandom random = new SecureRandom();
	    KeyGenerator keyGenerator = KeyGenerator.getInstance(mechanismName);
	    keyGenerator.initialize(random);
	    Key sessionKey = keyGenerator.generateKey();

	    System.out.println("initialize Cipher.");
	    initCipher(sessionKey, mode);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws Exception {
	try {
	    this.ioStream = ioStream;
	    String[] ownModePreferenceList = (String[])ownMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String[] partnerModePreferenceList = (String[])partnerMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String mode = (SSONETConnection.computePreferred(RatedString.toRatedString(ownModePreferenceList),
		RatedString.toRatedString(partnerModePreferenceList)))[0];
	    System.out.println("Using mode: " + mode);

	    StringTokenizer stringTokenizer = new StringTokenizer(getClass().getName(),".");
	    for (int i = 0; i<4; i++) {
		mechanismName = stringTokenizer.nextToken();
	    }
	    Key sessionKey = keyExchange.getSessionKey(mechanismName);
	    initCipher(sessionKey, mode);
	} catch (NullPointerException e) {
	    throw new ParameterNotCompatibleException();
	}
    }

    /** Initialize crypto mechanisms
    * @param sessionKey symmetric mechanism private key
    * @param mode mechanism mode
    */
    private void initCipher(Key sessionKey, String mode) {
	try {
	    cipherIn =  Cipher.getInstance(mechanismName + "/" + mode + "/OneAndZeroes");
	    cipherOut = Cipher.getInstance(mechanismName + "/" + mode + "/OneAndZeroes");

	    cipherIn.initDecrypt(sessionKey);
	    cipherOut.initEncrypt(sessionKey);

	    System.out.println(mechanismName + " instance initialized");
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    public MechanismConfiguration getDefaultConfiguration() {
	MechanismConfiguration  mechanismConfiguration = new MechanismConfiguration();
	String defaultMode[] = {getModes()[0]};
	mechanismConfiguration.put(KEY_MODE_PREFERENCELIST, defaultMode);
	mechanismConfiguration.put(PROVIDERLIST, defaultProvider);
	return mechanismConfiguration;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualMechanismConfiguration)
    {
        DetailConfigurationDialog detailConfigurationDialog = new DetailConfigurationDialog(parentFrame, getClass().getName());
        if(actualMechanismConfiguration == null) actualMechanismConfiguration = getDefaultConfiguration();
        detailConfigurationDialog.addTabProvider((String[])actualMechanismConfiguration.get(PROVIDERLIST));
        detailConfigurationDialog.addTabMode((String[])actualMechanismConfiguration.get(KEY_MODE_PREFERENCELIST),
	    SSONETConstants.CONFIDENTIALITY);
        detailConfigurationDialog.pack();
        detailConfigurationDialog.setResizable(false);
        detailConfigurationDialog.setLocationRelativeTo(parentFrame);
        detailConfigurationDialog.setVisible(true);

        if(detailConfigurationDialog.getReturnCode()) {
            actualMechanismConfiguration.put(KEY_MODE_PREFERENCELIST,
		detailConfigurationDialog.getModePreferenceList());
            actualMechanismConfiguration.put(PROVIDERLIST,
		detailConfigurationDialog.getProviderPreferenceList());
        }
        return actualMechanismConfiguration;
    }
}
