/*
 * smi/mechanisms/AbstractIntegrityMechanism.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import xjava.security.Cipher;
import java.security.Key;
import xjava.security.KeyGenerator;
import java.security.SecureRandom;
import java.util.StringTokenizer;

import javax.swing.JFrame;

import smi.baseConf.RatedString;
import smi.io.IOStream;
import smi.mechanisms.MechanismConfiguration;

import ssonet.KeyExchange;
import ssonet.SSONETConnection;
import ssonet.SSONETConstants;

import sun.security.x509.X509Cert;

public class AbstractIntegrityMechanism implements IOStream, Mechanism {
    protected IOStream ioStream;
    protected Cipher CipherIn, CipherOut;
    protected static String PROVIDERLIST = "Providerlist";
    protected static String KEY_MODE_PREFERENCELIST = "Mode.Preferencelist";
    protected static String KEY_KEYLENGTH = "KeyLength";
    protected static String KEY_ROUNDS = "Rounds";
    protected static String KEY_VARIANT = "Variant";
    protected String mechanismName;
    String defaultProvider[] = {"Cryptix"};

    public String[] getModes() {
	String[] modes = {"CBC","CFB","PCBC"};
	return modes;
    }

    public int available() {
	return ioStream.available();
    }

    public void close() {
	ioStream.close();
    }

    public byte[] testEncrypt(byte[] buffer)
    {
	SignatureObject signatureObject = new SignatureObject(buffer);
	return CipherOut.crypt(signatureObject.getObjectAsByteArray());
    }

    public byte[] testDecrypt(byte[] buffer)
    {
	SignatureObject signatureObject = new SignatureObject(buffer);
	return CipherIn.crypt(signatureObject.getObjectAsByteArray());
    }

    public Object readObject() throws Exception {
	try {
	    SignatureObject signatureObject = (SignatureObject)ioStream.readObject();
	    byte[] MAC = new byte[CipherOut.blockSize()];
	    byte[] buffer = CipherIn.crypt(signatureObject.getObjectAsByteArray());
	    System.arraycopy(buffer, buffer.length-MAC.length, MAC,0, MAC.length);
	    byte[] verify = signatureObject.getSignature();
	    if(MAC.length == verify.length) {
		for(int i = 0; i<MAC.length; i++)
		    if(MAC[i] != verify[i]) throw new Exception();
		return signatureObject.getObject();
	    }
	    throw new Exception();
	}
	catch (Exception e) {
	    throw e;
	}
    }

    public void writeObject(Object o) throws Exception {
	try {
	    SignatureObject signatureObject = new SignatureObject(o);
	    byte[] MAC = new byte[CipherOut.blockSize()];
	    byte[] buffer = CipherOut.crypt(signatureObject.getObjectAsByteArray());
	    System.arraycopy(buffer, buffer.length-MAC.length, MAC,0, MAC.length);
	    signatureObject.setSignature(MAC);
	    ioStream.writeObject(signatureObject);
	}
	catch (Exception e) {
	    throw e;
	}
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {
	    this.ioStream = ioStream;
	    String[] modePreferenceList = (String[])ownMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String mode = modePreferenceList[0];
	    System.out.println("Using mode: " + mode);

	    StringTokenizer stringTokenizer = new StringTokenizer(getClass().getName(),".");
	    for (int i = 0; i < 4; i++) {
		mechanismName = stringTokenizer.nextToken();
	    }

	    System.out.println("generating " + mechanismName + " Key.");
	    SecureRandom random = new SecureRandom();
	    KeyGenerator keyGenerator = KeyGenerator.getInstance(mechanismName);
	    keyGenerator.initialize(random);
	    Key sessionKey = keyGenerator.generateKey();

	    System.out.println("initialize Cipher.");
	    initCipher(sessionKey, mode);
	}
	catch (Exception e) {
	    System.out.println("Error in AbstractIntegrityMechansism.init");
	    e.printStackTrace();
	}
    }


    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws Exception {
	try {
	    this.ioStream = ioStream;

	    String[] ownModePreferenceList = (String[])ownMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String[] partnerModePreferenceList = (String[])partnerMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String mode = (SSONETConnection.computePreferred(RatedString.toRatedString(ownModePreferenceList),
		RatedString.toRatedString(partnerModePreferenceList)))[0];
	    System.out.println("Using mode: " + mode);

	    StringTokenizer stringTokenizer = new StringTokenizer(getClass().getName(),".");
	    for (int i = 0; i < 4; i++) {
		mechanismName = stringTokenizer.nextToken();
	    }
	    System.out.println("mech name "+mechanismName);

	    Key sessionKey = keyExchange.getSessionKey(mechanismName);
	    initCipher(sessionKey, mode);
	}
	catch (Exception e) {
	    System.out.println("Error in AbstractIntegrityMechansism.init");
	    e.printStackTrace();
	    throw new ParameterNotCompatibleException();
	}
    }

    /** Initialize crypto mechanisms
    * @param sessionKey symmetric mechanism private key
    * @param mode mechanism mode
    */
    private void initCipher(Key sessionKey, String mode) {
	try {
	    CipherIn =  Cipher.getInstance(mechanismName + "/" + mode + "/OneAndZeroes");
	    CipherOut = Cipher.getInstance(mechanismName + "/" + mode + "/OneAndZeroes");

	    CipherIn.initEncrypt(sessionKey);
	    CipherOut.initEncrypt(sessionKey);

	    System.out.println(mechanismName + " instance initialized");
	}
	catch(Exception e) {
	    System.out.println("Error in AbstractIntegrityMechansism.initCipher");
	    e.printStackTrace();
	}
    }

    public MechanismConfiguration getDefaultConfiguration()
    {
        MechanismConfiguration  mechanismConfiguration = new MechanismConfiguration();
        String defaultMode[] = {getModes()[0]};
        mechanismConfiguration.put(KEY_MODE_PREFERENCELIST, defaultMode);
        mechanismConfiguration.put(PROVIDERLIST, defaultProvider);
        return mechanismConfiguration;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualMechanismConfiguration)
    {
        DetailConfigurationDialog detailConfigurationDialog = new DetailConfigurationDialog(parentFrame, getClass().getName());
        if(actualMechanismConfiguration == null)
            actualMechanismConfiguration = getDefaultConfiguration();
        detailConfigurationDialog.addTabProvider((String [])actualMechanismConfiguration.get(PROVIDERLIST));
        detailConfigurationDialog.addTabMode((String [])actualMechanismConfiguration.get(KEY_MODE_PREFERENCELIST),
	    SSONETConstants.INTEGRITY);
        detailConfigurationDialog.pack();
        detailConfigurationDialog.setResizable(false);
        detailConfigurationDialog.setLocationRelativeTo(parentFrame);
        detailConfigurationDialog.setVisible(true);
        if(detailConfigurationDialog.getReturnCode())
            actualMechanismConfiguration.put(KEY_MODE_PREFERENCELIST,
	    detailConfigurationDialog.getModePreferenceList());
	actualMechanismConfiguration.put(PROVIDERLIST,
	    detailConfigurationDialog.getProviderPreferenceList());
        return actualMechanismConfiguration;
    }
}
