/*
 * smi/mechanisms/accountability/RSA.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.accountability;

import cryptix.provider.rsa.RawRSAPublicKey;
import cryptix.provider.rsa.RawRSAPrivateKey;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ByteArrayInputStream;
import java.security.KeyPair;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.util.StringTokenizer;
import java.util.ResourceBundle;

import javax.swing.JFrame;
import javax.swing.JPanel;

import smi.awt.JPreferenceList;
import smi.baseConf.RatedString;
import smi.io.IOStream;
import smi.mechanisms.AbstractAccountabilityMechanism;
import smi.mechanisms.DetailConfigurationDialog;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.Mechanism;
import smi.mechanisms.ParameterNotCompatibleException;
import smi.baseConf.BaseConfiguration;
import ssonet.KeyExchange;
import ssonet.SSONETConnection;

import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

/** RSA-Mechanism
*/

public class RSA extends AbstractAccountabilityMechanism {
    private JPreferenceList variantPreferenceList;
    private static String variants[] = {"MD-2","MD-5","RIPEMD-160","SHA-1"};
    private static String KEY_VARIANT = "KeyVariante";

    public RSA() {
	mechanismName = "RSA";
	mechanismCertificateFileName = new String(System.getProperty("ssonet_cfg","")
	+"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSACertificate.der");
	System.out.println(mechanismName + " Sign instance created");
    }

    /** returns the default configuration of RSA
    * @return default configuration of RSA
    */
    public MechanismConfiguration getDefaultConfiguration() {
	MechanismConfiguration  defaultMechanismConfiguration = new MechanismConfiguration();
	defaultMechanismConfiguration.put(KEY_KEYLENGTH, new Integer(512));
	String [] defaultVariant = {"SHA-1"};
	String defaultProvider[] = {"Cryptix"};
	defaultMechanismConfiguration.put(PROVIDERLIST, defaultProvider);
	defaultMechanismConfiguration.put(KEY_VARIANT, defaultVariant);
	return defaultMechanismConfiguration;
    }

    /** creates a dialog for mechanism configuration
    * @param parentFrame parent frame
    * @param actualConfiguration actual mechanism configuration
    * @return MechanismConfiguration new mechanism configuration
    */
    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualConfiguration) {
	this.parentFrame = parentFrame;
	detailConfigurationDialog = new DetailConfigurationDialog(parentFrame,"RSA");

	if(actualConfiguration == null) actualConfiguration = getDefaultConfiguration();

	// providers
	detailConfigurationDialog.addTabProvider((String[])actualConfiguration.get(PROVIDERLIST));

	// key length
	Integer actualKeyLength = (Integer)actualConfiguration.get(KEY_KEYLENGTH);
	if (actualKeyLength == null) actualKeyLength = new Integer(512);
	detailConfigurationDialog.addTabKey(512,1024,8, actualKeyLength.intValue());

	// variants
	detailConfigurationDialog.addTab(BaseConfiguration.getLangResources().getString("VARIANTS"), createVariantTab((String[])actualConfiguration.get(KEY_VARIANT)));
	// certificate
	detailConfigurationDialog.addTab(BaseConfiguration.getLangResources().getString("CERTIFICATE"), certificateTab());
	detailConfigurationDialog.pack();
	detailConfigurationDialog.setResizable(false);
	detailConfigurationDialog.setLocationRelativeTo(parentFrame);
	detailConfigurationDialog.setVisible(true);
	if(detailConfigurationDialog.getReturnCode()) {
	    actualConfiguration.put(PROVIDERLIST, detailConfigurationDialog.getProviderPreferenceList());
	    actualConfiguration.put(KEY_KEYLENGTH, new Integer(detailConfigurationDialog.getKeyLength()));
	    actualConfiguration.put(KEY_VARIANT, variantPreferenceList.getPreferenceListItems());
	}
	return actualConfiguration;
    }

    /** reads an RSA public key
    * @return X509Key public RSA key
    */
    protected X509Key readMechanismPublicKey() {
	X509Key publicKey = null;
	try {
	    System.out.println("reading public key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSAPublicKey.der");
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(fileInputStream);
	    byte[] rawKeyData = rawRSAPublicKey.getEncoded();
	    publicKey = createX509Key("MD5/RSA", rawKeyData);
	} catch(Exception e) {
	    e.printStackTrace();
	}
	return publicKey;
    }

    /** writes an RSA public key into a file
    * @param publicKey public RSA key
    */
    protected void writeMechanismPublicKey(PublicKey publicKey) {
	try {
	    createKeyDirectory();
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSAPublicKey.der");
	    System.out.println("writing public key to accountability/RSAPublicKey.der");
	    fileOutputStream.write(publicKey.getEncoded());
	    fileOutputStream.close();
	} catch(Exception e) {
	    System.out.println("Error in RSA.writeMechanismPublicKey");
	    e.printStackTrace();
	}
    }

    /** writes an RSA private key into a file
    * @param privateKey private RSA key
    */
    protected void writeMechanismPrivateKey(PrivateKey privateKey) {
	try {
	    createKeyDirectory();
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSAPrivateKey.der");
	    System.out.println("writing private key to accountability/RSAPrivateKey.der");
	    fileOutputStream.write(privateKey.getEncoded());
	    fileOutputStream.close();
	} catch(Exception e) {
	    System.out.println("Error in RSA.writeMechanismPrivateKey");
	    e.printStackTrace();
	}
    }

    /** creates the panel "Varinat"
    * @param actualVariants actual variants
    */
    private JPanel createVariantTab(String[] actualVariant) {
	JPanel variantPanel = new JPanel();

	variantPreferenceList = new JPreferenceList(5,
	    BaseConfiguration.getLangResources().getString("PREFERENCE LIST"),
	    BaseConfiguration.getLangResources().getString("AVAILABLE VARIANTS"),
	    actualVariant,
	    variants);
	variantPanel.add(variantPreferenceList);

	return variantPanel;
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {

	    String[] ownVariants = (String[])ownMechanismConfiguration.get(KEY_VARIANT);
	    String variant = ownVariants[0];
	    System.out.println("Using variant: " + variant);

	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSAPublicKey.der");
	    RawRSAPublicKey testKey = new RawRSAPublicKey(fileInputStream);
	    initCipher(testKey, variant, ioStream);
	}
	catch (Exception e) {
	    System.out.println("Error in  RSA.init");
	    e.printStackTrace();
	}
    }

    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws ParameterNotCompatibleException {
	try {
	    String[] ownVariants = (String[])ownMechanismConfiguration.get(KEY_VARIANT);
	    String[] partnerVariants = (String[])partnerMechanismConfiguration.get(KEY_VARIANT);
	    String variant = (SSONETConnection.computePreferred(RatedString.toRatedString(ownVariants),
		RatedString.toRatedString(partnerVariants)))[0];
	    System.out.println("Using variant: " + variant);

	    X509Cert mechanismCertificate = keyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSACertificate.der");
	    PublicKey testKey = keyExchange.getPartnerMechanismKey(mechanismCertificate);

	    initCipher(testKey, variant, ioStream);
	}
	catch (Exception e) {
	    System.out.println("Error in RSA.init");
	    e.printStackTrace();
	    throw new ParameterNotCompatibleException();
	}
    }

    /** Initialize crypto mechanism
    * @param testKey test key
    * @param variant used hash algorithm
    * @param ioStream IOStream
    */
    private void initCipher(PublicKey testKey, String variant, IOStream ioStream) {
	try {
	    this.ioStream = ioStream;

	    byte[] rawBuffer;
	    if (testKey instanceof RawRSAPublicKey) {
		rawBuffer = testKey.getEncoded();
	    } else {
		rawBuffer = KeyExchange.DERtoRAW(testKey.getEncoded());
	    }
	    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(rawBuffer);
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(byteArrayInputStream);

	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/RSAPrivateKey.der");
	    RawRSAPrivateKey signKey = new RawRSAPrivateKey(fileInputStream);
	    fileInputStream.close();

	    signatureIn = Signature.getInstance(variant + "/" + mechanismName);
	    signatureOut = Signature.getInstance(variant + "/" + mechanismName);

	    signatureIn.initVerify(rawRSAPublicKey);
	    signatureOut.initSign(signKey);
	    System.out.println(mechanismName + " instance initialized");
	}
	catch(Exception e) {
	    System.out.println("Error in RSA.initCipher");
	    e.printStackTrace();
	}
    }
}
