/*
 * smi/mechanisms/confidentiality/RSA.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.confidentiality;

import cryptix.provider.rsa.RawRSAPublicKey;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.File;
import xjava.security.Cipher;
import xjava.security.Mode;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import smi.baseConf.NewCertificateDialog;
import smi.baseConf.BaseConfiguration;
import smi.io.IOStream;
import smi.mechanisms.AbstractConfidentialityMechanism;
import smi.mechanisms.DetailConfigurationDialog;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.ParameterNotCompatibleException;

import ssonet.KeyExchange;

import sun.security.x509.AlgorithmId;
import sun.security.util.DerValue;
import sun.security.util.DerOutputStream;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;
import sun.security.util.BigInt;

public class RSA extends AbstractConfidentialityMechanism implements ActionListener {
    private ResourceBundle resource = BaseConfiguration.getLangResources();
    // private JButton certificateButton = new JButton("refresh Certificate");
    // private JButton keyButton = new JButton("refresh key");
    private JButton certificateButton = new JButton(resource.getString("REFRESH CERTIFICATE"));
    private JButton keyButton = new JButton(resource.getString("REFRESH KEY"));
    private JLabel titleLabel, dateLabel;
    private JFrame parentFrame;
    private DetailConfigurationDialog detailConfigurationDialog;
    private X509Cert mechanismCertificate;

    public RSA() {
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {
	    mechanismCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSACertificate.der");
	    PublicKey publicKey = mechanismCertificate.getPublicKey();

	    initCipher(publicKey, ioStream);
	} catch (Exception e) {
	    System.out.println("Error in RSA.init");
	    e.printStackTrace();
	}
    }


    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws ParameterNotCompatibleException {
	try {
	    mechanismCertificate = keyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSACertificate.der");
	    PublicKey publicKey = keyExchange.getPartnerMechanismKey(mechanismCertificate);

	    initCipher(publicKey, ioStream);
	} catch (Exception e) {
	    System.out.println("Error in RSA.init");
	    e.printStackTrace();
	    throw new ParameterNotCompatibleException();
	}
    }

    /** Initialize crypto mechanisms
    * @param publicKey public RSA key
    * @param keyExchange prepared protocol
    */
    private void initCipher(PublicKey publicKey, IOStream ioStream) {
	try {
	    this.ioStream = ioStream;
	    PrivateKey privateKey = KeyExchange.readRSAPrivateKey(System.getProperty("ssonet_cfg","")
		+"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSAPrivateKey.der");
	    byte[] rawBuffer = KeyExchange.DERtoRAW(publicKey.getEncoded());
	    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(rawBuffer);
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(byteArrayInputStream);

	    cipherIn = Cipher.getInstance("RSA/ECB/OneAndZeroes");
	    cipherOut = Cipher.getInstance("RSA/ECB/OneAndZeroes");
	    cipherIn.initDecrypt(privateKey);
	    cipherOut.initEncrypt(rawRSAPublicKey);
	    System.out.println("RSA instance initialized");
	} catch(Exception e) {
	    System.out.println("Error in RSA.initCipher");
	    e.printStackTrace();
	}
    }

    public MechanismConfiguration getDefaultConfiguration() {
	MechanismConfiguration defaultConfiguration = new MechanismConfiguration();
	String defaultProvider[] = {"Cryptix"};
	defaultConfiguration.put(PROVIDERLIST, defaultProvider);
	defaultConfiguration.put(KEY_KEYLENGTH, new Integer(512));
	return defaultConfiguration;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration aktConf) {
	this.parentFrame = parentFrame;
	detailConfigurationDialog = new DetailConfigurationDialog(parentFrame,"RSA");
	if(aktConf == null) aktConf = getDefaultConfiguration();

	// providers
	detailConfigurationDialog.addTabProvider((String[])aktConf.get(PROVIDERLIST));

	// key length
	Integer keyLength = (Integer)aktConf.get(KEY_KEYLENGTH);
	if (keyLength == null) keyLength = new Integer(512);
	detailConfigurationDialog.addTabKey(512,4096,512, keyLength.intValue());

	// certificate
	// detailConfigurationDialog.addTab("Certificate", certificateTab());
	detailConfigurationDialog.addTab(resource.getString("CERTIFICATE"), certificateTab());
	detailConfigurationDialog.pack();
	detailConfigurationDialog.setResizable(false);
	detailConfigurationDialog.setLocationRelativeTo(parentFrame);
	detailConfigurationDialog.setVisible(true);

	if(detailConfigurationDialog.getReturnCode()) {
	    aktConf.put(PROVIDERLIST, detailConfigurationDialog.getProviderPreferenceList());
	    aktConf.put(KEY_KEYLENGTH, new Integer(detailConfigurationDialog.getKeyLength()));
	}
	return aktConf;
    }

    private JPanel certificateTab() {
	JPanel certificatePanel = new JPanel();
	certificateButton.addActionListener(this);
	keyButton.addActionListener(this);

	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	certificatePanel.setLayout(gridBagLayout);

	mechanismCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	+"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSACertificate.der");
	if (mechanismCertificate == null) {
	    titleLabel = new JLabel (resource.getString("NO CERTIFICATE PRESENT"), JLabel.CENTER);
	    dateLabel = new JLabel(resource.getString("PLEASE CREATE A CERTIFICATE"), JLabel.CENTER);
	} else {
	    titleLabel = new JLabel (resource.getString("THIS CERTIFICATE IS VALID TO"), JLabel.CENTER);
	    dateLabel = new JLabel(mechanismCertificate.getNotAfter().toString(), JLabel.CENTER);
	}

	gridBagConstraints.insets.top = 10;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.gridheight = 1;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.weighty = .25;
	gridBagLayout.setConstraints(titleLabel, gridBagConstraints);
	certificatePanel.add(titleLabel);

	gridBagConstraints.insets.top = 0;
	gridBagConstraints.gridy = 1;
	gridBagConstraints.weighty = .25;
	gridBagLayout.setConstraints(dateLabel, gridBagConstraints);
	certificatePanel.add(dateLabel);

	gridBagConstraints.gridy = 2;
	gridBagLayout.setConstraints(certificateButton, gridBagConstraints);
	certificatePanel.add(certificateButton);

	gridBagConstraints.gridy = 3;
	gridBagConstraints.insets.bottom = 10;
	gridBagLayout.setConstraints(keyButton, gridBagConstraints);
	certificatePanel.add(keyButton);

	return certificatePanel;
    }

    public void actionPerformed(ActionEvent actionEvent) {
	Object object = actionEvent.getSource();
	if(object == certificateButton) {
	    createNewCertificate();
	} else if(object == keyButton) {
	    createKeyPair();
	    createNewCertificate();
	}
    }

    private void createNewCertificate() {
	try {
	    System.out.println("reading sign key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeDSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privateKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    System.out.println("reading sign certificate");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeDSACertificate.der");
	    X509Cert issuerCertificate = new X509Cert();
	    issuerCertificate.decode(fileInputStream);
	    fileInputStream.close();

	    System.out.println("reading public key");

	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSAPublicKey.der");
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(fileInputStream);
	    byte[] rawKeyData = rawRSAPublicKey.getEncoded();
	    X509Key publicKey = createX509Key("MD5/RSA", rawKeyData);

	    X500Signer signer = issuerCertificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);
	    X500Name subjectName = issuerCertificate.getSubjectName();

	    GregorianCalendar gregorianCalendar = (GregorianCalendar)Calendar.getInstance();
	    Date notBefore = gregorianCalendar.getTime();

	    int duration = new Integer(NewCertificateDialog.getAnswer(parentFrame)).intValue();
	    gregorianCalendar.add(Calendar.DAY_OF_YEAR, duration);
	    Date notAfter = gregorianCalendar.getTime();

	    X509Cert certificate = new X509Cert(subjectName, publicKey, notBefore, notAfter);

	    System.out.println("sign certificate");
	    SecureRandom randomGenerator = new SecureRandom();
	    byte[] randomBuffer = new byte[16];
	    randomGenerator.nextBytes(randomBuffer);
	    X509Cert signedCertificate = new X509Cert(certificate.encodeAndSign(new BigInt(randomBuffer), signer));
	    System.out.println("Certificate:\n" + signedCertificate.toString());

	    System.out.println("writing Certificate");
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSACertificate.der");
	    signedCertificate.encode(fileOutputStream);
	    fileOutputStream.close();

	    // titleLabel.setText("This certificate is valid up to:");
	    titleLabel.setText(resource.getString("THIS CERTIFICATE IS VALID TO"));
	    dateLabel.setText(signedCertificate.getNotAfter().toString());
	} catch(Exception ex) {
	    System.out.println(ex);
	}
    }

    /**
     * Create directory for confidentiality keys, if it does not exist and check writability
     */
    private void createKeyDirectory() throws Exception{
	File acDir = new File(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality");
	if(!acDir.exists()){
	    acDir.mkdir();
	}
	if(!acDir.canWrite()) throw new Exception("Directory for keys is not writable.");
    }

    private void createKeyPair() {
	try {
	    createKeyDirectory();
	    KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA");
	    int keyLength = detailConfigurationDialog.getKeyLength();
	    System.out.println("initialize KeyPairGenerator for " + keyLength + " Bit KeyPair");
	    keyPairGenerator.initialize(keyLength);
	    System.out.println("generate KeyPair.");
	    KeyPair ownKeyPair = keyPairGenerator.generateKeyPair();

	    System.out.println("public key:\n" + ownKeyPair.getPublic().toString());
	    System.out.println("\nprivate key:\n" + ownKeyPair.getPrivate().toString());

	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSAPublicKey.der");
	    System.out.println("writing public key to confidentiality/RSAPublicKey.der");
	    fileOutputStream.write(ownKeyPair.getPublic().getEncoded());
	    fileOutputStream.close();

	    fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSAPrivateKey.der");
	    System.out.println("writing private key to confidentiality/RSAPrivateKey.der");
	    fileOutputStream.write(ownKeyPair.getPrivate().getEncoded());
	    fileOutputStream.close();
	} catch(Exception e) {
	    System.out.println("Error in RSA.createKeyPair");
	    e.printStackTrace();
	}
    }

    private X509Key createX509Key(String algorithmID, byte[] rawKeyData) {
	try {
	    // insert leading zero
	    byte[] rawKeyData2 = new byte[rawKeyData.length + 1];
	    rawKeyData2[0] = 0;
	    for (int i = 0; i < rawKeyData.length; i++) rawKeyData2[i + 1] = rawKeyData[i];

	    byte[] algIDArray = smi.util.Emit.emit(algorithmID);

	    DerValue algDerValue = new DerValue(algIDArray);
	    DerValue keyDerValue = new DerValue(DerValue.tag_BitString, rawKeyData2);

	    DerValue[] both = {algDerValue, keyDerValue};
	    DerOutputStream derOutputStream2 = new DerOutputStream();
	    derOutputStream2.putSequence(both);
	    byte[] bothArray = derOutputStream2.toByteArray();
	    DerValue bothDerValue = new DerValue(bothArray);

	    return (X509Key) X509Key.parse(bothDerValue);
	} catch(Exception e) {
	    System.out.println("Error in RSA.createX509Key");
	    e.printStackTrace();
	}
	return null;
    }
}
