package smi.user;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;
import java.util.*;
import java.io.*;
import java.security.*;

import sun.security.util.DerValue;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import sun.security.util.BigInt;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;

import cryptix.provider.rsa.RawRSAPublicKey;

import ssonet.KeyExchange;
import ssonet.IdentityCheck;
import smi.baseConf.InvalidCertificateException;
import smi.baseConf.NewCertificateDialog;
import smi.baseConf.CancelException;

/**
 * Title:        SSONET_N
 * Description:  SSONET - Sicherheit und Schutz in offenen Netzen
 * fuer Client-Server-Verbindungen
 * Copyright:    Copyright (c) 2000
 * Company:      TU Dresden
 * Dialog to display and change user properties, especially keys and certificates
 * Provider Cryptix must be preloaded to use this class.
 * @author Sebastian Clau
 * @version 1.0
 */

public class UserPropertiesDialog extends JDialog implements ActionListener {
    private JLabel userIDLabel, keysLabel, languageLabel, directoryLabel, keysStatusLabel,
	certificatesLabel, certificatesStatusLabel;
    private JTextField userIDField, directoryField;
    private JButton createKeysButton, applyButton, cancelButton, testCertificatesButton, createCertificatesButton;
    private JComboBox languageBox;

    private Locale locale;
    private ResourceBundle resource;
    private boolean isNewUser;

    private String userDir;
    private Hashtable userProperties;
    public static String USER_PROPERTIES_USERID = "USERID";
    public static String USER_PROPERTIES_LANGUAGE = "LANGUAGE";
    private PrivateKey sigPrivKey;
    private PrivateKey ciphPrivKey;
    private X509Cert sigCert;
    private X509Cert ciphCert;
    private int keysStatus;
    private int certificatesStatus;
    private static final int STATE_NOT_AVAILABLE = 0;
    private static final int STATE_OK = 1;
    private static final int STATE_FALSE = 2;

    private boolean debug;

    /**
     *  reads user-properties and creates the dialog
     *  @param userDir directory name of the user directory (only last part)
     *  @param isNewUser true, if a new User is to be created
     */
    public UserPropertiesDialog(JDialog owner, boolean modal, Locale locale,
				String userDir, boolean isNewUser) {
	super(owner, modal);

	// debug-mode
	try{
	    debug = false;
	    if(System.getProperty("debug") != null){
		debug = true;
	    }
	}
	catch (Exception e){};

	this.locale = locale;
	this.userDir = userDir;
	this.isNewUser = isNewUser;
	resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", locale);
	if(debug) System.out.println("Land,Sprache:"+resource.getLocale().getCountry()+", "
	    + resource.getLocale().getLanguage());
	if(debug) System.out.println("IsNewUser: "+isNewUser);
	// initialize Userdata
	userProperties = new Hashtable();
	if(!isNewUser){
	    // read userdata from disk and check it
	    try{
		FileInputStream fis = new FileInputStream(System.getProperty("ssonet_cfg")+"users/"
		    +userDir + "/userproperties.dat");
		ObjectInputStream ois = new ObjectInputStream(fis);
		userProperties = (Hashtable)ois.readObject();
		ois.close();
		fis.close();
	    }
	    catch (Exception e){
		// File not found or wrong form -- leave field empty
	    }
	    // read keys from disk and check them
	    if(checkKeys()){
		keysStatus = STATE_OK;
	    }else{
		keysStatus = STATE_FALSE;
	    }
	}

	//create window elements
	setTitle(resource.getString("USER PROPERTIES"));

	getContentPane().setLayout(new BorderLayout());

	JPanel dataPanel = new JPanel();
	GridBagLayout gl = new GridBagLayout();
	dataPanel.setLayout(gl);
	getContentPane().add(dataPanel, BorderLayout.CENTER);
	dataPanel.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(), resource.getString("USER DATA")));

	GridBagConstraints gc = new GridBagConstraints();

	userIDLabel = new JLabel(resource.getString("USER ID")+":");
	gc.gridx = 0;
	gc.gridy = 0;
	gc.insets = new Insets(3,10,3,10);
	gc.fill = GridBagConstraints.NONE;
	gl.setConstraints(userIDLabel, gc);
	dataPanel.add(userIDLabel);

	userIDField = new JTextField(20);
	if(userProperties.get(USER_PROPERTIES_USERID) != null){
	    userIDField.setText((String)userProperties.get(USER_PROPERTIES_USERID));
	}
	gc.gridx = 1;
	gc.gridwidth = 1;
	gc.fill = GridBagConstraints.HORIZONTAL;
	gl.setConstraints(userIDField, gc);
	dataPanel.add(userIDField);

	keysLabel = new JLabel(resource.getString("KEYS")+":");
	gc.gridx = 0;
	gc.gridy = 1;
	gc.gridwidth = 1;
	gl.setConstraints(keysLabel, gc);
	dataPanel.add(keysLabel);

	keysStatusLabel = new JLabel();
	setKeysStatus(keysStatus);
	gc.gridx = 1;
	gl.setConstraints(keysStatusLabel, gc);
	dataPanel.add(keysStatusLabel);

	createKeysButton = new JButton(resource.getString("CREATE KEYS AND CERTIFICATES"));
	gc.gridx = 3;
	gl.setConstraints(createKeysButton, gc);
	dataPanel.add(createKeysButton);

	certificatesLabel = new JLabel(resource.getString("CERTIFICATES"));
	gc.gridx = 0;
	gc.gridy = 2;
	gl.setConstraints(certificatesLabel, gc);
	dataPanel.add(certificatesLabel);

	certificatesStatusLabel = new JLabel();
	setCertificatesStatus(certificatesStatus);
	gc.gridx = 1;
	gl.setConstraints(certificatesStatusLabel, gc);
	dataPanel.add(certificatesStatusLabel);

	testCertificatesButton = new JButton(resource.getString("TEST CERTIFICATES"));
	gc.gridx = 2;
	gl.setConstraints(testCertificatesButton, gc);
	dataPanel.add(testCertificatesButton);

	createCertificatesButton = new JButton(resource.getString("CREATE CERTIFICATES"));
	gc.gridx = 3;
	gl.setConstraints(createCertificatesButton, gc);
	dataPanel.add(createCertificatesButton);

	languageLabel = new JLabel(resource.getString("LANGUAGE"));
	gc.gridx = 0;
	gc.gridy = 3;
	gl.setConstraints(languageLabel, gc);
	dataPanel.add(languageLabel);

	String[] languages = {"Deutsch", "English"};
	// 0: German
	// 1: English
	languageBox = new JComboBox(languages);
	languageBox.setEditable(false);
	if(userProperties.get(USER_PROPERTIES_LANGUAGE) != null){
	    try{
		languageBox.setSelectedIndex(((Integer)userProperties.get(USER_PROPERTIES_LANGUAGE)).intValue());
	    }
	    catch(Exception e){
		e.printStackTrace();
	    }
	}
	gc.gridx = 1;
	gc.gridwidth = 1;
	gl.setConstraints(languageBox, gc);
	dataPanel.add(languageBox);

	directoryLabel = new JLabel(resource.getString("DIRECTORY"));
	gc.gridx = 0;
	gc.gridy = 4;
	gl.setConstraints(directoryLabel, gc);
	dataPanel.add(directoryLabel);

	directoryField = new JTextField(20);
	directoryField.setText(userDir);
	if(isNewUser){
	    directoryField.setEditable(true);
	}else{
	    directoryField.setEditable(false);
	}
	gc.gridx = 1;
	gc.gridwidth = 1;
	gl.setConstraints(directoryField, gc);
	dataPanel.add(directoryField);

	JPanel buttonPanel = new JPanel();
	getContentPane().add(buttonPanel, BorderLayout.SOUTH);
	GridBagLayout glb = new GridBagLayout();
	buttonPanel.setLayout(glb);

	applyButton = new JButton(resource.getString("APPLY"));
	gc.gridx = 0;
	gc.gridy = 0;
	gc.gridwidth = 1;
	gc.anchor = GridBagConstraints.CENTER;
	gc.fill = GridBagConstraints.NONE;
	glb.setConstraints(applyButton, gc);
	buttonPanel.add(applyButton);

	cancelButton = new JButton(resource.getString("CANCEL"));
	gc.gridx = 1;
	gl.setConstraints(cancelButton, gc);
	buttonPanel.add(cancelButton);

	if(isNewUser){
	    createKeysButton.setEnabled(false);
	    createCertificatesButton.setEnabled(false);
	    testCertificatesButton.setEnabled(false);
	}

	// include actionListener
	createKeysButton.addActionListener(this);
	testCertificatesButton.addActionListener(this);
	createCertificatesButton.addActionListener(this);
	applyButton.addActionListener(this);
	cancelButton.addActionListener(this);

	pack();
    }

    //XXXXXXXXXXXXXXXXXXXXXXXXXXXXX Hier gehts weiter XXXXXXXXXXXXXXXXXXXXXXXXXXX
	// (Umgebungsvariable ssonet_user einfhren, wie neues SSONET)
	//Die neuen Dialoge eingliedern (am Anfang der Basekonfiguration die Nutzerauswahl,
	// statt dem Schlsselmen in der Baseconfig den Properties-Dialog
	// Skripte updaten, damit Nutzerauswahl aus Skripten mglich ist


    /**
     * react to pressed buttons
     */
    public void actionPerformed(ActionEvent ae){
	if(ae.getSource() == createKeysButton){
	    //create new keys and Certificates for the user
	    if(debug) System.out.println("Create keys button pressed.");
	    setCursor(new Cursor(Cursor.WAIT_CURSOR));
	    boolean OK = true;
	    if(!createNewCipherKeyPair()) OK = false;
	    if(OK){
		if(!createNewSignatureKeyPair()) OK = false;
	    }
	    if(OK){
		JOptionPane.showMessageDialog(this, resource.getString("CREATE KEY MESSAGE")
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeDSAPrivateKey.object"
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeRSAPublicKey.object"
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeRSAPrivateKey.der"
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeRSAPublicKey.der",
		    resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
	    }else{
		JOptionPane.showMessageDialog(this, resource.getString("KEY CREATION FAILED"),
			resource.getString("ERROR"), JOptionPane.ERROR_MESSAGE);
	    }
	    if(checkKeys()){
		setKeysStatus(STATE_OK);
		if(debug)System.out.println("KeysStatus: OK");
	    }else{
		setKeysStatus(STATE_FALSE);
		if(debug)System.out.println("KeysStatus: ERROR");
	    }
	    // create new certificates for the user
	    if(OK){
		if(!createNewSignatureCertificate()) OK = false;
	    }
	    if(OK){
		if(!createNewCipherCertificate()) OK = false;
	    }
	    if(OK && checkCertificates()){
		setCertificatesStatus(STATE_OK);
		JOptionPane.showMessageDialog(this, resource.getString("CREATE CERTIFICATES MESSAGE")
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeDSACertificate.der"
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeRSACertificate.der",
		    resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
	    }else{
		setCertificatesStatus(STATE_FALSE);
		JOptionPane.showMessageDialog(this, resource.getString("CREATE CERTIFICATES FAILED"),
		    resource.getString("ERROR"), JOptionPane.INFORMATION_MESSAGE);
	    }
	    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}
	else if (ae.getSource() == testCertificatesButton){
	    // test the users certificates
	    this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
	    if(checkCertificates()) {
		setCertificatesStatus(STATE_OK);
	    } else {
		setCertificatesStatus(STATE_FALSE);
	    }
	    this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}
	else if (ae.getSource() == createCertificatesButton){
	    //create new certificates
	    setCursor(new Cursor(Cursor.WAIT_CURSOR));
	    boolean OK = true;
	    if(!createNewSignatureCertificate()) OK = false;
	    if(OK){
		if(!createNewCipherCertificate()) OK = false;
	    }
	    if(OK && checkCertificates()){
		setCertificatesStatus(STATE_OK);
		JOptionPane.showMessageDialog(this, resource.getString("CREATE CERTIFICATES MESSAGE")
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeDSACertificate.der"
		    +"\n"+System.getProperty("ssonet_cfg")+"users/"+userDir+"/KeyExchangeRSACertificate.der",
		    resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
	    }else{
		setCertificatesStatus(STATE_FALSE);
		JOptionPane.showMessageDialog(this, resource.getString("CREATE CERTIFICATES FAILED"),
		    resource.getString("ERROR"), JOptionPane.INFORMATION_MESSAGE);
	    }
	    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}
	else if (ae.getSource() == applyButton){
	    setCursor(new Cursor(Cursor.WAIT_CURSOR));
	    if(isNewUser){
		// create userdir and userdata
		try{
		    // check, if user directory allready exists and if it is writable
		    userDir = directoryField.getText();
		    File dir = null;
		    try{
			dir = new File(System.getProperty("ssonet_cfg")+"users/"+userDir);
		    }catch (Exception e){
		    }
		    if(dir.exists()){
			if(!dir.isDirectory()) throw new Exception("Could not create user directory.");
			if(!dir.canWrite()) throw new Exception("User directory is write-protected.");
		    } else {
			//create user directory
			if(!dir.mkdirs()) throw new Exception("Could not create user directory.");
			File subdir = new File(System.getProperty("ssonet_cfg")+"users/"+userDir+"/accountability");
			if(!subdir.mkdir()) throw new Exception("Could not create directory for accountability keys.");
			subdir = new File(System.getProperty("ssonet_cfg")+"users/"+userDir+"/confidentiality");
			if(!subdir.mkdir()) throw new Exception("Could not create directory for confidentiality keys.");
		    }
		    // write userdata
		    userProperties.put(USER_PROPERTIES_USERID,userIDField.getText());
		    userProperties.put(USER_PROPERTIES_LANGUAGE, new Integer(languageBox.getSelectedIndex()));
		    FileOutputStream fos = new FileOutputStream(System.getProperty("ssonet_cfg")+"users/"
			+ userDir + "/userproperties.dat");
		    ObjectOutputStream oos = new ObjectOutputStream(fos);
		    oos.writeObject(userProperties);
		    oos.close();
		    fos.close();
		    // create keys and certificates
		    if(!createNewCipherKeyPair()) throw new Exception("Could not create cipher keypair.");
		    if(!createNewSignatureKeyPair()) throw new Exception("Could not create signature keypair.");
		    if(!createNewSignatureCertificate()) throw new Exception("Could not create signature certificate.");
		    if(!createNewCipherCertificate()) throw new Exception("Could not create cipher certificate.");
		    JOptionPane.showMessageDialog(this, "User data created successfully.",
			resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
		}
		catch(Exception e){
		    JOptionPane.showMessageDialog(this, "Error while creating user data:\n\n"+ e.getMessage(),
			resource.getString("ERROR"), JOptionPane.ERROR_MESSAGE);
		}
	    } else {
		// apply changes to userID and language
		userProperties.put(USER_PROPERTIES_USERID,userIDField.getText());
		userProperties.put(USER_PROPERTIES_LANGUAGE, new Integer(languageBox.getSelectedIndex()));
		try{
		    FileOutputStream fos = new FileOutputStream(System.getProperty("ssonet_cfg")+"users/"
			+ userDir + "/userproperties.dat");
		    ObjectOutputStream oos = new ObjectOutputStream(fos);
		    oos.writeObject(userProperties);
		    oos.close();
		    fos.close();
		}
		catch (Exception e){
		    JOptionPane.showMessageDialog(this, "Could not write changes.",
			resource.getString("ERROR"), JOptionPane.ERROR_MESSAGE);
		}
	    }
	    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	    dispose();
	}
	else if (ae.getSource() == cancelButton){
	    dispose();
	}
    }

    /**
     * set state label for user keys
     */
    private void setKeysStatus(int state){
	this.keysStatus = state;
	switch (state){
	case STATE_NOT_AVAILABLE:
	    keysStatusLabel.setText(resource.getString("N/A"));
	    break;
	case STATE_OK:
	    keysStatusLabel.setText(resource.getString("OK"));
	    break;
	case STATE_FALSE:
	    keysStatusLabel.setText(resource.getString("ERROR"));
	};
    }

    /**
     * set state label for user certificates
     */
    private void setCertificatesStatus(int state){
	this.certificatesStatus = state;
	switch (state){
	case STATE_NOT_AVAILABLE:
	    certificatesStatusLabel.setText(resource.getString("N/A"));
	    break;
	case STATE_OK:
	    certificatesStatusLabel.setText(resource.getString("OK"));
	    break;
	case STATE_FALSE:
	    certificatesStatusLabel.setText(resource.getString("ERROR"));
	};
    }

    /**
     * Checks, if there are allready keys
     * @return true, if there are Keys, false otherwise
     */
    private boolean checkKeys(){
	sigPrivKey = KeyExchange.readPrivateKey(System.getProperty("ssonet_cfg")+"users/"+userDir
	    +"/KeyExchangeDSAPrivateKey.object");
	ciphPrivKey = KeyExchange.readRSAPrivateKey(System.getProperty("ssonet_cfg")+"users/"+userDir
	    +"/KeyExchangeRSAPrivateKey.der");
	if(debug) System.out.println("Schlssel = null?"+(sigPrivKey == null)+","
	    + (ciphPrivKey == null));
	if((sigPrivKey == null) || (ciphPrivKey == null)){
	    return false;
	}
	return true;
    }

    /**
     * Checks, if Certificates are valid
     * @return true, if the Certificates are valid, false otherwise
     */
    private boolean checkCertificates(){
	try {
	    JFrame dummyFrame = new JFrame();

	    if(debug) System.out.println("reading trustcenter certificate:");
	    X509Cert trustCenterCertificate
		= KeyExchange.readCertificate(System.getProperty("ssonet_cfg")+"trustcenter/TrustCenterCertificate.der");
	    if(debug) System.out.println(trustCenterCertificate.toString());
	    PublicKey trustCenterPublicKey = trustCenterCertificate.getPublicKey();

	    if(debug) System.out.println("testing trustcenter certificate.");
	    if (!KeyExchange.testCertificate(trustCenterCertificate, trustCenterPublicKey)){
		if(debug) System.out.println("TrustCenter Certificate not valid!");
		throw new InvalidCertificateException();
	    }
	    if (!IdentityCheck.getAnswer("TrustCenter certificate", trustCenterCertificate, dummyFrame)){
		if(debug) System.out.println("TrustCenter Certificate not valid!");
		throw new InvalidCertificateException();
	    }

	    if(debug) System.out.println("reading testCertificate:");
	    X509Cert testCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg")+"users/"
		+ userDir + "/KeyExchangeDSACertificate.der");
	    if(debug) System.out.println(testCertificate.toString());
	    PublicKey ownPublicKey = testCertificate.getPublicKey();

	    if(debug) System.out.println("testing testCertificate.");
	    if (!KeyExchange.testCertificate(testCertificate, trustCenterPublicKey)){
		if(debug) System.out.println("Certificate for TestKey not valid!");
		throw new InvalidCertificateException();
	    }
	    if (!IdentityCheck.getAnswer("Certificate for sign key", testCertificate, dummyFrame)){
		if(debug) System.out.println("Certificate for TestKey not valid!");
		throw new InvalidCertificateException();
	    }

	    if(debug) System.out.println("reading cipherCertificate:");
	    X509Cert cipherCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg")+"users/"
		+ userDir + "/KeyExchangeRSACertificate.der");
	    if(debug) System.out.println(cipherCertificate.toString());

	    if(debug) System.out.println("testing cipherCertificate.");
	    if (!KeyExchange.testCertificate(cipherCertificate, ownPublicKey)){
		if(debug)System.out.println("Certificate for CipherKey not valid!");
		throw new InvalidCertificateException();
	    }
	    if (!IdentityCheck.getAnswer("Certificate for chipher key", cipherCertificate, dummyFrame)){
		if(debug) System.out.println("Certificate for CipherKey not valid!");
		throw new InvalidCertificateException();
	    }
	    JOptionPane.showMessageDialog(this, resource.getString("VALID KEY MESSAGE"),
		resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
	    return true;
	}
	catch (InvalidCertificateException ice) {
	    JOptionPane.showMessageDialog(this, resource.getString("INVALID KEY MESSAGE"),
		resource.getString("WARNING"), JOptionPane.ERROR_MESSAGE);
	    return false;
	}
	catch (Exception e){
	    JOptionPane.showMessageDialog(this, resource.getString("CERTIFICATE NOT FOUND"),
		resource.getString("WARNING"), JOptionPane.ERROR_MESSAGE);
	    return false;
	}
    }

    /**
     * Creates new cipher keypair for the user (RSA)
     */
    private boolean createNewCipherKeyPair(){
	try {
	    KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA","Cryptix");
	    if(debug) System.out.println("initialize KeyPairGenerator for 1024 Bit KeyPair");
	    System.out.println("SecurityProvider:"+keyPairGenerator.getProvider().getInfo());
	    keyPairGenerator.initialize(1024);
	    if(debug) System.out.println("generate KeyPair.");
	    KeyPair ownKeyPair = keyPairGenerator.generateKeyPair();

	    if(debug) System.out.println("public key:\n" + ownKeyPair.getPublic().toString());
	    if(debug) System.out.println("\nprivate key:\n" + ownKeyPair.getPrivate().toString());

	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeRSAPublicKey.der");
	    if(debug) System.out.println("writing public key to keys/KeyExchangeRSAPublicKey.der");
	    fileOutputStream.write(ownKeyPair.getPublic().getEncoded());
	    fileOutputStream.close();

	    fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeRSAPrivateKey.der");
	    if(debug)System.out.println("writing private key to keys/KeyExchangeRSAPrivateKey.der");
	    fileOutputStream.write(ownKeyPair.getPrivate().getEncoded());
	    fileOutputStream.close();
	    return true;
	} catch(Exception e) {
	    e.printStackTrace();
	}
	return false;
    }

    /**
     * Creates new signature keypair for the user (DSA)
     */
    private boolean createNewSignatureKeyPair(){
	try {
	    KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("DSA");
	    if(debug)System.out.println("initialize KeyPairGenerator.");
	    keyPairGenerator.initialize(1024);
	    if(debug)System.out.println("generate KeyPair.");
	    KeyPair ownKeyPair = keyPairGenerator.generateKeyPair();

	    if(debug) System.out.println("public key:\n"+ownKeyPair.getPublic().toString());
	    if(debug) System.out.println("\nprivate key:\n"+ownKeyPair.getPrivate().toString());

	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSAPublicKey.der");
	    if(debug) System.out.println("writing public key to File: KeyExchangeDSAPublicKey.der");
	    fileOutputStream.write(ownKeyPair.getPublic().getEncoded());
	    fileOutputStream.close();

	    fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSAPublicKey.object");
	    if(debug) System.out.println("writing public key to File: KeyExchangeDSAPublicKey.object");
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(fileOutputStream);
	    objectOutputStream.writeObject(ownKeyPair.getPublic());
	    fileOutputStream.close();
	    objectOutputStream.close();

	    fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSAPrivateKey.der");
	    if(debug) System.out.println("writing private key to File: KeyExchangeDSAPrivateKey.der");
	    fileOutputStream.write(ownKeyPair.getPrivate().getEncoded());
	    fileOutputStream.close();

	    fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSAPrivateKey.object");
	    if(debug) System.out.println("writing private key to File: KeyExchangeDSAPrivateKey.object");
	    objectOutputStream = new ObjectOutputStream(fileOutputStream);
	    objectOutputStream.writeObject(ownKeyPair.getPrivate());
	    fileOutputStream.close();
	    objectOutputStream.close();
	    return true;
	}
	catch(java.lang.Exception ex) {
	    System.out.println(ex);
	}
	return false;
    }

    /**
     * Creates new cipher certificate for the user.
     * The certificate is created with the signkey of the user.
     */
    private boolean createNewCipherCertificate(){
	try {
	    if(debug) System.out.println("reading sign key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privateKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    if(debug) System.out.println("reading sign certificate");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSACertificate.der");
	    X509Cert issuerCertificate = new X509Cert();
	    issuerCertificate.decode(fileInputStream);
	    fileInputStream.close();

	    X500Signer signer = issuerCertificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);
	    X500Name subjectName = issuerCertificate.getSubjectName();

	    GregorianCalendar gregorianCalendar = (GregorianCalendar)Calendar.getInstance();
	    Date notBefore = gregorianCalendar.getTime();

	    String durString = NewCertificateDialog.getAnswer(new JFrame(), resource.getString("TESTKEY CERTIFICATE"));
	    if (durString == null) throw new CancelException();
	    int duration = Integer.parseInt(durString);
	    gregorianCalendar.add(Calendar.DAY_OF_YEAR, duration);
	    Date notAfter = gregorianCalendar.getTime();

	    if(debug) System.out.println("reading public key");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeRSAPublicKey.der");
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(fileInputStream);
	    byte[] rawKeyData = rawRSAPublicKey.getEncoded();
	    X509Key publicKey = createRSAX509Key(rawKeyData);

	    X509Cert certificate = new X509Cert(subjectName, publicKey, notBefore, notAfter);

	    if (debug) System.out.println("sign certificate");
	    SecureRandom randomGenerator = new SecureRandom();
	    byte[] randomBuffer = new byte[16];
	    randomGenerator.nextBytes(randomBuffer);
	    X509Cert signedCertificate = new X509Cert(certificate.encodeAndSign(new BigInt(randomBuffer), signer));
	    if(debug) System.out.println("Certifikat:\n" + signedCertificate.toString());

	    if(debug) System.out.println("writing Certificate");
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeRSACertificate.der");
	    certificate.encode(fileOutputStream);
	    fileOutputStream.close();
	    return true;
	} catch(CancelException ce) {
	} catch(Exception ex) {
	    ex.printStackTrace();
	}
	return false;
    }

    /**
     * Creates a RSA-X509-key from a raw rsa key data
     * (a hack due to incompatibilities of Cryptix)
     */
     private X509Key createRSAX509Key(byte[] rawKeyData) {
	try {
	    // insert leading zero
	    byte[] rawKeyData2 = new byte[rawKeyData.length + 1];
	    rawKeyData2[0] = 0;
	    for (int i = 0; i < rawKeyData.length; i++) rawKeyData2[i + 1] = rawKeyData[i];

	    byte[] algIDArray = smi.util.Emit.emit("MD5/RSA");

	    DerValue algDerValue = new DerValue(algIDArray);
	    DerValue keyDerValue = new DerValue(DerValue.tag_BitString, rawKeyData2);

	    DerValue[] both = {algDerValue, keyDerValue};
	    DerOutputStream derOutputStream2 = new DerOutputStream();
	    derOutputStream2.putSequence(both);
	    byte[] bothArray = derOutputStream2.toByteArray();
	    DerValue bothDerValue = new DerValue(bothArray);

	    return (X509Key)X509Key.parse(bothDerValue);
	} catch(Exception e) {
	    e.printStackTrace();
	}
	return null;
    }

    /**
     * Create a new certificate for the Testkey of the user.
     * XXXX (must normally be done at a CA, but this time no CA is available,
     * so we do CA-Things here)
     */
    private boolean createNewSignatureCertificate(){
	try {
	    if(debug) System.out.println("reading trustcenter private key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"trustcenter/TrustCenterDSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privateKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    if(debug) System.out.println("reading testkey");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSAPublicKey.object");
	    objectInputStream = new ObjectInputStream(fileInputStream);
	    X509Key publicKey = (X509Key)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    X500Name subjectName = new X500Name(userIDField.getText(), "SSONET-User", "TU-Dresden", "DE");
	    GregorianCalendar gregorianCalendar = (GregorianCalendar)Calendar.getInstance();
	    Date notBefore = gregorianCalendar.getTime();
	    String durString = NewCertificateDialog.getAnswer(new JFrame(), resource.getString("CIPHERKEY CERTIFICATE"));
	    if (durString == null) throw new CancelException();
	    int duration = Integer.parseInt(durString);
	    gregorianCalendar.add(Calendar.DAY_OF_YEAR, duration);
	    Date notAfter = gregorianCalendar.getTime();

	    if(debug) System.out.println("create certificate.");

	    X509Cert certificate = new X509Cert(subjectName, publicKey, notBefore, notAfter);
	    if(debug) System.out.println("reading issuer certificate");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"trustcenter/TrustCenterCertificate.der");
	    X509Cert trustCenterCertificate = new X509Cert();
	    trustCenterCertificate.decode(fileInputStream);
	    fileInputStream.close();
	    X500Signer signer = trustCenterCertificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);

	    if(debug) System.out.println("sign certificate");
	    X509Cert signedCertificate = new X509Cert(certificate.encodeAndSign(new BigInt(1234567890), signer));
	    if(debug) System.out.println("Certifikate:\n"+signedCertificate.toString());

	    if(debug) System.out.println("writing Certificate into File.");
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")
		+"users/"+userDir+"/KeyExchangeDSACertificate.der");
	    signedCertificate.encode(fileOutputStream);
	    fileOutputStream.close();
	    return true;
	}
	catch(Exception e) {
	    if(debug) {
	      System.out.println(e);
	      e.printStackTrace();
	    }
	}
	return false;
    }

    // dummy-main only temporary
    public static void main(String[] argv){
	JDialog upd = new UserPropertiesDialog(new JDialog(), true, Locale.GERMAN,
				"kunde", true);
	upd.show();
    }
}