/*
 * ssonet/KeyExchange.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import cryptix.provider.key.RawSecretKey;
import cryptix.provider.rsa.RawRSAPrivateKey;
import cryptix.provider.rsa.RawRSAPublicKey;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.math.BigInteger;
import xjava.security.Cipher;
import java.security.Key;
import xjava.security.KeyGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.util.Date;

import javax.swing.JFrame;

import smi.io.IOStream;

import sun.security.x509.X509Cert;
import sun.security.util.DerValue;
import sun.security.x509.AlgorithmId;

/**
* Protocol for secure key exchange
*/

public class KeyExchange {
    private boolean debug;

    private BigInteger random;
    private Key sessionKey;
    private X509Cert ownCipherCertificate;
    private PublicKey ownTestKey,ownCipherKey,partnerTestKey,partnerCipherKey;
    private PrivateKey privateKey,signKey;
    private IOStream ioStream;
    private boolean firstKeyExchange = true;
    private boolean isServer;
    private byte[] seed = {1,2,3,4,5,6,7,8,9,0};
    private SecureRandom secureRandom = new SecureRandom();

    /**
    * Protocol constructor
    * @param ioStream algorithm IOStream
    * @param parentFrame parent frame
    * @param ownTestCertificate test key certificate issued by a trustcenter
    * @param ownCipherCertificate self issued cipher key certificate
    * @param trustCenterTestKey trustcenter test key
    * @param isServer determines, if protocol is used for a client or a server
    * @exception SSONETCertificateNotValidException if partner certificate not valid
    */
    public KeyExchange(IOStream ioStream,
	JFrame parentFrame,
	X509Cert ownTestCertificate,
	X509Cert ownCipherCertificate,
	PublicKey trustCenterTestKey,
	boolean isServer) throws Exception{
	// debug-mode
	debug = false;
	try{
	    if(System.getProperty("debug") != null){
		debug = true;
	    }
	}catch (Exception e){};

	this.ioStream = ioStream;
	this.ownCipherCertificate = ownCipherCertificate;
	this.isServer = isServer;

	ownTestKey = ownTestCertificate.getPublicKey();
	ownCipherKey = ownCipherCertificate.getPublicKey();

	try {
	    // exchange test certificates
	    if(debug) System.out.println("Write testkey certificate");
	    ioStream.writeObject(ownTestCertificate);
	    if(debug) System.out.println("Wait for partners testkey certificate");
	    X509Cert partnerTestCertificate = (X509Cert)ioStream.readObject();
	    if(debug) System.out.println("test partners certificate");
	    if (testCertificate(partnerTestCertificate, trustCenterTestKey))
		partnerTestKey = partnerTestCertificate.getPublicKey();
	    else {
		if(debug) System.out.println ("certificate NOT valid.");
		throw new SSONETCertificateNotValidException();
	    }

	    if(debug) System.out.println ("check partner identity certificate");
	    if (!isServer)
	    {
		if (!IdentityCheck.getAnswer("Communication partner certificate", partnerTestCertificate, parentFrame))
		{
		    if(debug) System.out.println ("partner identity certificate NOT valid.");
		    throw new SSONETCertificateNotValidException();
		}
	    }
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeDSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    signKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();
	} catch (SSONETCertificateNotValidException se) {
	    if(debug) System.out.println ("Some key certificates not valid. Abort.");
	    throw se;
	} catch (Exception e) {
	    if(debug) e.printStackTrace();
	    throw e;
	}
	privateKey = readRSAPrivateKey(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeRSAPrivateKey.der");
    }

    /** exchanges mechanism key certificates and extracts keys
    * @param ownMechanismCertificate selfissued mechanism key certificate
    * @return PublicKey communication partners public mechanism key
    * @exception Exception if partners mechanism certificate not valid
    */
    public PublicKey getPartnerMechanismKey(X509Cert ownMechanismCertificate)throws Exception {
	PublicKey partnerMechanismKey = null;
	try {
	    if(debug) System.out.println("Write mechanism public key certificate");
	    ioStream.writeObject(ownMechanismCertificate);
	    if(debug) System.out.println("Wait for partners mechanism public key certificate");
	    X509Cert partnerMechanismCertificate = (X509Cert)ioStream.readObject();
	    if (testCertificate(partnerMechanismCertificate, partnerTestKey))
		partnerMechanismKey = partnerMechanismCertificate.getPublicKey();
	    else throw new Exception();
	} catch(java.lang.Exception e) {
	    if(debug) e.printStackTrace();
	    throw e;
	}
	return partnerMechanismKey;
    }

    /** Keyexchangeprotocol for symmetric algorithms
    * -> first key exchange:
    *       -Server sends a random number and a cipher key certificate
    *       -Client tests the certificate, creates a sessionkey, signs sessionkey and
    *        random number, encrypts sessionkey and signature
    *   -Server decrypts the message, tests the signature and extracts the sessionkey
    * -> all following key exchanges:
    *       -Client creates a sessionkey, signs sessionkey and random number,
    *           encrypts sessionkey and signature
    *   -Server decrypts the message, tests the signature and extracts the sessionkey
    * @param mechanismName session key mechanism name
    * @return Key Sessionkey
    * @exception Exception at server site if signature under sessionkey and random number not valid
    *                      at client site if servers cipher key certificate not valid
    */
    public Key getSessionKey(String mechanismName)throws Exception {
	try {
	    if(isServer) {
		if (firstKeyExchange) {
		    if(debug) System.out.println("create random number");
		    byte[] randomBuffer = new byte[16];
		    secureRandom.nextBytes(randomBuffer);
		    random = new BigInteger(randomBuffer);
		    if(debug) System.out.println("write random number");
		    ioStream.writeObject(random);
		    if(debug) System.out.println("write cipher key certificate");
		    ioStream.writeObject(ownCipherCertificate);
		    firstKeyExchange = false;
		}

		if(debug) System.out.println("getSessionKey: Wait for encrypted message");
		byte[] encryptedMessage = (byte[])ioStream.readObject();

		if(debug) System.out.println("getSessionKey: decrypt message");
		byte[] plainText = decrypt(encryptedMessage, privateKey);
		int keyLength = plainText[0] | (plainText[1]<<8);

		if(debug) System.out.println("extract " + keyLength + " Byte Sessionkey");
		byte[] keyBuffer = new byte[keyLength];
		int offset = 2;
		for (int i=0; i<keyLength; i++) {
		    keyBuffer[i] = plainText[i + offset];
		}
		sessionKey = new RawSecretKey(mechanismName, keyBuffer);

		if(debug) System.out.println("test signatur under sessionkey and random number");
		byte[] signatureBuffer = new byte[plainText.length-2-keyLength];
		offset = 2 + keyLength;
		for (int i=0; i<signatureBuffer.length; i++) {
		    signatureBuffer[i] = plainText[i + offset];
		}
		byte[] buffer = joinByteArrays(sessionKey.getEncoded(), random.toByteArray());
		if (!verify(buffer, signatureBuffer, partnerTestKey)) {
		    if(debug) System.out.println("signature under sessionkey and random number not valid!");
		    throw new Exception();
		}
	    } else {
		if (firstKeyExchange) {
		    if(debug) System.out.println("Wait for random number");
		    random = (BigInteger)ioStream.readObject();

		    if(debug) System.out.println("Wait for partner cipher key certificate");
		    X509Cert partnerCipherCertificate = (X509Cert)ioStream.readObject();

		    System.out.println("test certificate");
		    if (testCertificate(partnerCipherCertificate, partnerTestKey)) {
			partnerCipherKey = partnerCipherCertificate.getPublicKey();
		    } else
			throw new Exception();
		    firstKeyExchange = false;
		}

		if(debug) System.out.println("create sessionkey");
		KeyGenerator keyGenerator = KeyGenerator.getInstance(mechanismName);
		keyGenerator.initialize(secureRandom);
		sessionKey = keyGenerator.generateKey();
		if(debug) System.out.println(sessionKey.getEncoded().length + " Byte Sessionkey created");

		if(debug) System.out.println("Sign sessionkey and random number:");
		byte[] signBuffer = joinByteArrays(sessionKey.getEncoded(), random.toByteArray());
		byte[] signature = sign(signBuffer, signKey);

		if(debug) System.out.println("encrypt message");
		byte[] keyLength = new byte[2];
		keyLength[0] = (byte)(sessionKey.getEncoded().length&0x00FF);
		keyLength[1] = (byte)((sessionKey.getEncoded().length>>8)&0xFF);

		byte[] buffer = joinByteArrays(keyLength, sessionKey.getEncoded());
		byte[] plainMessage = joinByteArrays(buffer, signature);
		byte[] encryptedMessage = encrypt(plainMessage, partnerCipherKey);

		if(debug) System.out.println("write encrypted message");
		ioStream.writeObject(encryptedMessage);
	    }
	} catch(Exception e) {
	    if(debug) e.printStackTrace();
	    throw new Exception();
	}
	return sessionKey;
    }

    /** encrypt ByteArray with public key
    * @param input data to encrypt
    * @param publicKey public key
    * @return byte[] encrypted data
    */
    private byte[] encrypt(byte[] input, PublicKey publicKey) {
	try {
	    Cipher cipherOut=Cipher.getInstance("RSA/ECB/OneAndZeroes");

	    byte[] rawBuffer = DERtoRAW(publicKey.getEncoded());
	    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(rawBuffer);
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(byteArrayInputStream);

	    cipherOut.initEncrypt(rawRSAPublicKey);
	    byte[] ouput=cipherOut.crypt(input);
	    return ouput;
	} catch(java.lang.Exception e) {
	    if(debug) {
		System.out.println("Error in Keyexchange.encrypt");
		e.printStackTrace();
	    }
	}
	return null;
    }

    /** decrypt ByteArray with private key
    * @param input data to decrypt
    * @param privateKey private key
    * @return byte[] decrypted data
    */
    private byte[] decrypt(byte[] input, PrivateKey privateKey) {
	try {
	    Cipher cipherIn=Cipher.getInstance("RSA/ECB/OneAndZeroes");
	    cipherIn.initDecrypt(privateKey);
	    byte[] output=cipherIn.crypt(input);
	    return output;
	} catch(java.lang.Exception e) {
	    if(debug) {
		System.out.println("Error in Keyexchange.decrypt");
		e.printStackTrace();
	    }
	}
	return null;
    }

    /** signs ByteArray with private key
    * @param input data to sign
    * @param privateKey private key
    * @return byte[] signature
    */
    private byte[] sign(byte[] input, PrivateKey privateKey) {
	try {
	    Signature signatureInstance = Signature.getInstance("DSA");
	    signatureInstance.initSign(privateKey);
	    signatureInstance.update(input);
	    byte[] signature = signatureInstance.sign();
	    return signature;
	} catch(java.lang.Exception e) {
	    if(debug) {
		System.out.println("Error in Keyexchange.sign");
		e.printStackTrace();
	    }
	}
	return null;
    }

    /** tests signature under a message
    * @param message message
    * @param signature signature
    * @param publicKey public key
    * @return boolean true if signature valid, otherwise false
    */
    private boolean verify(byte[] message, byte[] signature, PublicKey publicKey) {
	try {
	    Signature signatureInstance = Signature.getInstance("DSA");
	    signatureInstance.initVerify(publicKey);
	    signatureInstance.update(message);
	    boolean value = signatureInstance.verify(signature);
	    return value;
	} catch(java.lang.Exception e) {
	    if(debug) {
		System.out.println("Error in Keyexchange.verify");
		e.printStackTrace();
	    }
	}
	return false;
    }

    /** tests if a certificate is valid
    * @param toTest certificate to test
    * @param publicKey certificate issuers public key
    * @return boolean true, if certificate valid, otherwise false
    */
    public static boolean testCertificate(X509Cert toTest, PublicKey publicKey) {
	try {
	    toTest.verify(publicKey);
	    Date now = new Date();
	    if (now.before(toTest.getNotBefore())) {
		System.out.println("Certificate is still not valid!");
		return false;
	    }
	    if (now.after(toTest.getNotAfter())) {
		System.out.println("Certificate is no longer valid!");
		return false;
	    }
	} catch(java.lang.Exception e) {
	    //System.out.println("Certificate is not valid!");
	    //e.printStackTrace();
	    return false;
	}
	return true;
    }

    /** reads a certificate from file
    * @param certificateFileName certificate file name
    * @return X509Cert read certificate
    */
    public static X509Cert readCertificate(String certificateFileName)
    {
	X509Cert x509Cert = null;
	try {
	    FileInputStream fileInputStream = new FileInputStream(certificateFileName);
	    x509Cert = new X509Cert();
	    x509Cert.decode(fileInputStream);
	    fileInputStream.close();
	} catch(Exception e) {
	    //System.out.println("Error in KeyExchange.readCertificate");
	    //e.printStackTrace();
	}
	return x509Cert;
    }

    /**
     * Reads a private Key from a File
     */
    public static PrivateKey readPrivateKey(String privateKeyFileName) {
	try {
	    FileInputStream fileInputStream = new FileInputStream(privateKeyFileName);
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();
	    return privKey;
	} catch(java.lang.Exception e) {
	    //System.out.println("Error in KeyExchange.readPrivateKey");
	    //e.printStackTrace();
	}
	return null;
    }

    /** reads a RSA private key from file
    * @param privateKeyFileName private key file name
    * @return PrivateKey read private key
    */
    public static PrivateKey readRSAPrivateKey(String privateKeyFileName) {
	try {
	    FileInputStream fileInputStream = new FileInputStream(privateKeyFileName);
	    PrivateKey privateKey = new RawRSAPrivateKey(fileInputStream);
	    fileInputStream.close();
	    return privateKey;
	} catch(java.lang.Exception e) {
	    //System.out.println("Error in KeyExchange.readRSAPrivateKey");
	    //e.printStackTrace();
	}
	return null;
    }

    /** joins two ByteArrays
    * @param array1 first ByteArray
    * @param array2 second ByteArray
    * @return byte[] Concatenation of both ByteArrays
    */
    private byte[] joinByteArrays(byte[] array1, byte[] array2) {
	try {
	    ByteArrayOutputStream byteArrayOutputStream =new ByteArrayOutputStream();
	    byteArrayOutputStream.write(array1,0, array1.length);
	    byteArrayOutputStream.write(array2,0, array2.length);
	    byte[] buffer = byteArrayOutputStream.toByteArray();
	    byteArrayOutputStream.close();
	    return buffer;
	} catch(java.io.IOException e) {
	    if(debug){
		System.out.println("Error in KeyExchange.joinByteArrays");
		e.printStackTrace();
	    }
	}
	return null;
    }

    /** extracts from DER-coded key data RAW-coded key data
    * @param input DER-coded key data
    * @return byte[] RAW-coded key data
    */
    public static byte[] DERtoRAW(byte[] input) {
	try {
	    DerValue derValue = new DerValue(input);
	    byte[] derValueByteArray = derValue.toByteArray();

	    DerValue[] derValuefield = derValue.data.getSequence(2);
	    byte[] byteArray = derValue.data.getBitString();

	    return byteArray;
	} catch(Exception e) {
	    System.out.println("Error in KeyExchange.DERtoRAW");
	    e.printStackTrace();
	}
	return null;
    }
}
