/*
 * ssonet/SSONETClientConnection.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.ConnectException;
import java.security.PublicKey;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;
import smi.baseConf.BaseConfiguration;
import smi.baseConf.RatedString;
import smi.io.IOStream;
import smi.io.ObjectIOStream;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.NoCommonMechanismException;
import smi.mechanisms.ConflictException;
import smi.mechanisms.MechanismInstance;

import ssonet.SSONETNegotiate;

import sun.security.x509.X509Cert;

import state.*;

public class SSONETClientConnection extends SSONETConnection {
    private int actionID;
    private ActionConfiguration ownActionConfiguration, partnerActionConfiguration;
    private Socket socket;
    private String partnerInternetAddress;
    private int port;
    private IOStream ioStream;
    private boolean interruptConnection;


    /** Creates a new SSONETClientConnection. If an error occures an exception will be thrown.
    * The actual connection will be created with connect.
    * @param applicationName Name of the applikation
    * @param actionID unique action number
    * @param parentFrame parent Frame
    * @param partnerInternetAddress Die Internet-Adresse des Partners
    * @param port Die Portnummer des Partners
    * @exception NullPointerException eigene Konfiguration konnte nicht geladen werden
    * @exception SecurityException Zertifikat wurde abgelehnt
    */
    public SSONETClientConnection(String applicationName,
	int actionID,
	JFrame parentFrame,
	String partnerInternetAddress,
	int port) throws IOException, NullPointerException, SSONETCertificateNotValidException {
        this.applicationName = applicationName;
        this.actionID = actionID;
        this.parentFrame = parentFrame;
        this.port = port;
        this.partnerInternetAddress = partnerInternetAddress;

        try {
            ownActionConfiguration = ApplicationConfiguration.getActionConfiguration(applicationName, actionID);
            if(ownActionConfiguration == null)throw new IOException();
            write2Tree(1, StateTreeConstants.INFORMATION,1, null,"Actionconfiguration for ID: " + actionID + " loaded.");
            initConnection();
	} catch (NullPointerException ne) {
            write2Tree(1, StateTreeConstants.CONFLICT,1, null,"Error while SSONETClientConnection initialization: Could not read Configuration");
	    throw ne;
	} catch (SSONETCertificateNotValidException cnve) {
            write2Tree(1, StateTreeConstants.CONFLICT,1, null,"Error while SSONETClientConnection initialization: Certificate not valid");
	    throw cnve;
        } catch (Exception e) {
            write2Tree(1, StateTreeConstants.CONFLICT,1, null,"Error while SSONETClientConnection initialization");
            e.printStackTrace();
	    throw new IOException();
        }
    }

    /** Create connection with communication partner. After creating the connection the
    * negotiation takes place. When it was successful an IOStream will be returned.
    * With readObject and writeObject one can communicate securely.
    * If the negotiation is not successfully, e.g. because the settings are not compatible
    * an exception will be thrown.
    * @return IOStream Input Output Stream
    * @exception IOException if settings are not compatible
    * @exception SSONETCloseConnectionException if connection forced to interrupt (cancel pressed)
    * @exception ConnectException if Host unreachable
    */
    public IOStream connect()
	throws 	IOException,
	SSONETCloseConnectionException,
	ConnectException,
	NoCommonMechanismException,
	ConflictException,
	SSONETCertificateNotValidException {
        try {
            interruptConnection = false; // on start, should not interrupt.
            write2Tree(1, StateTreeConstants.INFORMATION,1, null,"Try negotiation with: " + partnerInternetAddress);
            System.out.println("Try negotiation with: " + partnerInternetAddress);

            socket = new Socket(partnerInternetAddress, port);

            if (interruptConnection)
		throw new SSONETCloseConnectionException();

            InputStream inputStream = socket.getInputStream();
            OutputStream outputStream = socket.getOutputStream();

            write2Tree(StateTreeConstants.RESETCONFLICTS, StateTreeConstants.CONFLICT,1, null, null); //neue Aushandlung -> Konflikticon im state loeschen
            write2Tree(StateTreeConstants.NOTHING, StateTreeConstants.INFORMATION,1, null,"sending own configuration");
            write2Tree(StateTreeConstants.TCBEFORE, StateTreeConstants.USER,1,"Accountability", ownActionConfiguration.toString(SSONETConstants.ACCOUNTABILITY_SIGN));
            write2Tree(StateTreeConstants.TCBEFORE, StateTreeConstants.USER,1,"Anonymity", ownActionConfiguration.toString(SSONETConstants.ANONYMITY));
            write2Tree(StateTreeConstants.TCBEFORE, StateTreeConstants.USER,1,"Confidentality", ownActionConfiguration.toString(SSONETConstants.CONFIDENTIALITY));
            write2Tree(StateTreeConstants.TCBEFORE, StateTreeConstants.USER,1,"Integrity", ownActionConfiguration.toString(SSONETConstants.INTEGRITY));
            System.out.println("sending own configuration:\n" + ownActionConfiguration.toString());
            ownActionConfiguration.toStream(outputStream, true);

            partnerActionConfiguration = ActionConfiguration.fromStream(inputStream);
            write2Tree(StateTreeConstants.NOTHING, StateTreeConstants.INFORMATION,1, null,"got partner configuration");
            System.out.println("got partner configuration:\n" + partnerActionConfiguration.toString());

            if (interruptConnection)
		throw new SSONETCloseConnectionException();
            SSONETNegotiate.negotiatePhaseOne(ownActionConfiguration, partnerActionConfiguration);
            write2Tree(StateTreeConstants.NOTHING, StateTreeConstants.INFORMATION,1, null,"own configuration after phase 1");
            write2Tree(StateTreeConstants.TCAFTER, StateTreeConstants.USER,1,"Accountability", ownActionConfiguration.toString(SSONETConstants.ACCOUNTABILITY_SIGN));
            write2Tree(StateTreeConstants.TCAFTER, StateTreeConstants.USER,1,"Anonymity", ownActionConfiguration.toString(SSONETConstants.ANONYMITY));
            write2Tree(StateTreeConstants.TCAFTER, StateTreeConstants.USER,1,"Confidentality", ownActionConfiguration.toString(SSONETConstants.CONFIDENTIALITY));
            write2Tree(StateTreeConstants.TCAFTER, StateTreeConstants.USER,1,"Integrity", ownActionConfiguration.toString(SSONETConstants.INTEGRITY));
            System.out.println("own configuration after phase 1:\n" + ownActionConfiguration.toString());

            ownActionConfiguration.toStream(outputStream, true);
            partnerActionConfiguration = ActionConfiguration.fromStream(inputStream);
            System.out.println("partner configuration after phase 1:\n" + partnerActionConfiguration.toString());

            PhaseTwoResult phaseTwoResult = SSONETNegotiate.negotiatePhaseTwo(ownActionConfiguration,
		partnerActionConfiguration,
		settings);
            boolean conflictOccured = false;
            String errorMessage = new String();
	    String exceptionString = new String();
            // determining if any conflict occured....
            for (int i = 0; i < 5; i++) {
                if (phaseTwoResult.get(i) == SSONETConstants.CONFLICT)
		    conflictOccured = true;
	    }
            if (conflictOccured) {
                write2Tree(StateTreeConstants.NOTHING, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT,"Negotiation","Negotiation not successful");     // 1->fehlgeschlagen
                System.out.println("Negotiation not successful:");
                errorMessage = errorMessage + "Negotiation not successful:\n";
            }

            if (interruptConnection)
		throw new SSONETCloseConnectionException();
            // report details...
            if (phaseTwoResult.get(SSONETConstants.CONFIDENTIALITY) == SSONETConstants.CONFLICT) {
                write2Tree(StateTreeConstants.SZ, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT,"Confidentality","Conflict regarding Confidentiality!");
                System.out.println("-> Conflict regarding Confidentiality!");
                errorMessage = errorMessage + "-> Conflict regarding Confidentiality!\n";
		exceptionString += "Vertraulichkeit\n";
            }
            if (phaseTwoResult.get(SSONETConstants.ANONYMITY) == SSONETConstants.CONFLICT) {
                write2Tree(StateTreeConstants.SZ, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT,"Anonymity","Conflict regarding Anonymity!");
                System.out.println("-> Conflict regarding Anonymity!");
                errorMessage = errorMessage + "-> Conflict regarding Anonymity!\n";
		exceptionString += "Anonymitt\n";
            }
            if (phaseTwoResult.get(SSONETConstants.INTEGRITY) == SSONETConstants.CONFLICT) {
                write2Tree(StateTreeConstants.SZ, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT,"Integrity","Conflict regarding Integrity!");
                System.out.println("-> Conflict regarding Integrity!");
                errorMessage = errorMessage + "-> Conflict regarding Integrity!\n";
		exceptionString += "Integritt\n";
            }
            if (phaseTwoResult.get(SSONETConstants.ACCOUNTABILITY_SIGN) == SSONETConstants.CONFLICT) {
                write2Tree(StateTreeConstants.SZ, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT,"Accountability","Conflict regarding Accountability!");
                System.out.println("-> Conflict regarding Accountability!");
                errorMessage = errorMessage + "-> Conflict regarding Accountability!\n";
		exceptionString += "Zurechenbarkeit";
            }
            if (conflictOccured) {
                write2Tree(StateTreeConstants.NOTHING, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT,"No connection","No connection!");
                System.out.println("-> No connection!");
                errorMessage = errorMessage + "-> No connection!";
                throw new ConflictException (exceptionString);
            }

            ObjectIOStream objectIOStream = new ObjectIOStream(inputStream, outputStream);
            if (interruptConnection)
		throw new SSONETCloseConnectionException();

            PhaseTwoConfiguration ownPhaseTwoConfiguration = new PhaseTwoConfiguration(settings);
            if(settings.securityGoal[SSONETConstants.CONFIDENTIALITY]) {
                MechanismConfiguration[] configuration = new MechanismConfiguration[ownConfidencialityPreferenceList.length];
                for(int i = 0; i < ownConfidencialityPreferenceList.length; i++) {
                    configuration[i] =
   //nderung
			MechanismInstance.getMechanismConfiguration(SSONETConstants.CONFIDENTIALITY,
			ownConfidencialityPreferenceList[i].string);
                }
                ownPhaseTwoConfiguration.setConfidentiality(ownConfidencialityPreferenceList, configuration);
            }
            if(settings.securityGoal[SSONETConstants.ACCOUNTABILITY_SIGN]) {
                MechanismConfiguration[] configuration = new MechanismConfiguration[ownAccountabilityPreferenceList.length];
                for(int i = 0; i < ownAccountabilityPreferenceList.length; i++) {
                    configuration[i] =
   //nderung
			MechanismInstance.getMechanismConfiguration(SSONETConstants.ACCOUNTABILITY_SIGN,
			ownAccountabilityPreferenceList[i].string);
                }
                ownPhaseTwoConfiguration.setAccountability(ownAccountabilityPreferenceList, configuration);
            }

            if(settings.securityGoal[SSONETConstants.INTEGRITY]) {
                MechanismConfiguration[] configuration = new MechanismConfiguration[ownIntegrityPreferenceList.length];
                for(int i = 0; i < ownIntegrityPreferenceList.length; i++) {
    //nderung
                    configuration[i] = MechanismInstance.getMechanismConfiguration(SSONETConstants.INTEGRITY,
			ownIntegrityPreferenceList[i].string);
                }
                ownPhaseTwoConfiguration.setIntegrity(ownIntegrityPreferenceList, configuration);
            }

            objectIOStream.writeObject(ownPhaseTwoConfiguration);
            PhaseTwoConfiguration partnerPhaseTwoConfiguration = (PhaseTwoConfiguration)objectIOStream.readObject();
            System.out.println(partnerPhaseTwoConfiguration.toString());
            if (interruptConnection)
		throw new SSONETCloseConnectionException();

            IOStream ioStream = SSONETNegotiate.negotiatePhaseThree(ownPhaseTwoConfiguration,
		partnerPhaseTwoConfiguration,
		objectIOStream,
		parentFrame,
		trustCenterTestKey,
		ownTestCertificate,
		ownCipherCertificate,
		false);
            if(ioStream == null) {
                System.out.println("socket.close()");
                socket.close();
                throw new IOException();
            }
            System.out.println("successful");
            System.out.println(Integer.toString(actionID));
            write2Tree(1, StateTreeConstants.INFORMATION,1, null,"Negotiation successful: " + actionID);
            return (new SSONETStream(actionID, ioStream));
	} catch(SSONETCloseConnectionException se) {
	    close();
	    throw se;
	} catch (ConnectException ce) {
	    throw ce;
	} catch (NoCommonMechanismException me) {
	    throw me;
	} catch (ConflictException ce) {
	    throw ce;
	} catch (SSONETCertificateNotValidException cnve) {
	    throw cnve;
	} catch(Exception e) {
	    System.out.println("Error while negotitaing - Abort!");
	    e.printStackTrace();
	    throw new IOException();
	}
    }


    /**
    * Cause the connection to interrupt.
    */
    public void interrupt()
    {
	interruptConnection = true;
    }

    public void close() {
	try {
	    socket.close();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
