/*
 * ssonet/SSONETConnection.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.io.*;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.Socket;
import java.security.PublicKey;

import javax.swing.JFrame;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;
import smi.baseConf.BaseConfiguration;
import smi.baseConf.RatedString;
import smi.io.IOStream;
import smi.io.ObjectIOStream;
import smi.mechanisms.MechanismConfiguration;

import ssonet.SSONETNegotiate;

import sun.security.x509.X509Cert;

import state.Pad;

import state.StateTree;

public class SSONETConnection implements Serializable {
    protected JFrame parentFrame;
    protected String applicationName;
    /* jh5 - begin // 
	protected RatedString[] ownAnonymityPreferenceList;
	// jh5 - end */
    protected RatedString[] ownConfidencialityPreferenceList, ownAccountabilityPreferenceList, ownIntegrityPreferenceList;
    protected Settings settings;
    protected X509Cert ownTestCertificate, ownCipherCertificate;
    protected PublicKey trustCenterTestKey;

    protected boolean debug;

    public SSONETConnection() {
    }

    /** Initializes a new SSONETConnection. Own settings will be loaded.
    * When an error occures an exception is thrown.
    * @exception NullPointerException thrown, when configuration could not be read
    * @exception SecurityException certificate is not accepted by user
    */
    protected void initConnection() throws Exception {
	// debug-mode
	debug = false;
	try{
	    if(System.getProperty("debug") != null){
		debug = true;
	    }
	}
	catch (Exception e){};
	try{
	    settings = new Settings();

	    // Check system properties
	    if(System.getProperty("ssonet_cfg") == null){
		throw new NullPointerException("System-Property \"ssonet_cfg\" is not set.");
	    }
	    if(System.getProperty("ssonet_userdir") == null){
		throw new NullPointerException("System-Property \"ssonet_userdir\" is not set.");
	    }
	    if(debug) System.out.println("ssonet_cfg:"+System.getProperty("ssonet_cfg")+",ssonet_userdir:"
		+System.getProperty("ssonet_userdir"));

	    // read baseconfiguration
	    ownConfidencialityPreferenceList = BaseConfiguration.getPreferenceList(SSONETConstants.CONFIDENTIALITY);
	    if(ownConfidencialityPreferenceList == null)throw new NullPointerException();
	    writeStatus("PreferenceList: Confidentiality loaded.");

	    ownAccountabilityPreferenceList = BaseConfiguration.getPreferenceList(SSONETConstants.ACCOUNTABILITY_SIGN);
	    if(ownAccountabilityPreferenceList == null)throw new NullPointerException();
	    writeStatus("PreferenceList: Accountability loaded.");

	    ownIntegrityPreferenceList = BaseConfiguration.getPreferenceList(SSONETConstants.INTEGRITY);
	    if(ownIntegrityPreferenceList == null)throw new NullPointerException();
	    writeStatus("PreferenceList: Integrity loaded.");
		
		/* jh5 - begin //
	    ownAnonymityPreferenceList = BaseConfiguration.getPreferenceList(SSONETConstants.ANONYMITY);
	    if(ownAnonymityPreferenceList == null)throw new NullPointerException();
	    writeStatus("PreferenceList: Anonymity loaded.");
		// jh5 - end */

	    //read trustcenter and user certificates
	    if(debug) System.out.println("reading trustcenter certificate:");
	    X509Cert trustCenterCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg")
		+"trustcenter/TrustCenterCertificate.der");
	    trustCenterTestKey = trustCenterCertificate.getPublicKey();

	    if(debug) System.out.println("testing trustcenter certificate.");
	    if (!KeyExchange.testCertificate(trustCenterCertificate, trustCenterTestKey))
		throw new SecurityException();

	    if(debug) System.out.println("reading ownTestCertificate:");
	    ownTestCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg")
		+"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeDSACertificate.der");
	    PublicKey ownPublicKey = ownTestCertificate.getPublicKey();

	    if(debug) System.out.println("testing ownTestCertificate.");
	    if (!KeyExchange.testCertificate(ownTestCertificate, trustCenterTestKey))
		throw new SecurityException();

	    if(debug) System.out.println("reading ownCipherCertificate:");
	    ownCipherCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
		+"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeRSACertificate.der");

	    if(debug) System.out.println("testing ownCipherCertificate.");
	    if (!KeyExchange.testCertificate(ownCipherCertificate, ownPublicKey))
		throw new SecurityException();
	}
	catch (Exception e) {
	    throw e;
	}
    }

    public static String[] computePreferred(RatedString[] preferences1, RatedString[] preferences2) {
	boolean consensFound = false;
	int commonListIndex = 0;

	// check, if there are common items in list and put them in a new list
	RatedString commonList[] = new RatedString[preferences1.length];
	for(int i = 0; i < preferences1.length; i++)
	    for(int j = 0; j < preferences2.length; j++) {
	    if (preferences1[i].string.equals(preferences2[j].string)) {
		commonList[commonListIndex++] = new RatedString(preferences1[i].string,
		    preferences1[i].rating + preferences2[j].rating);
		consensFound = true;
	    }
	}
	if(!consensFound) return null;

	// reorder new list
	RatedString buffer;
	for(int j = 0; j < commonListIndex - 1; j++)
	    for(int i = j + 1; i < commonListIndex; i++) {
	    if(commonList[j].rating < commonList[i].rating) {
		buffer = commonList[i];
		commonList[i] = commonList[j];
		commonList[j] = buffer;
	    }
	}

	// create an array of Strings without the rating
	String resultStringArray[] = new String[commonListIndex];
	for(int i = 0; i < commonListIndex; i++)
	    resultStringArray[i] = commonList[i].string;
	return resultStringArray;
    }



    protected void writeStatus(String string) {
	try {
	    String stringToDisplay = applicationName + ": " + string + "\n";
	    byte[] bytesToDisplay = new byte[stringToDisplay.length() + 1];
	    bytesToDisplay[0] = (byte)state.Pad.SSONET;
	    byte[] stringBytes = stringToDisplay.getBytes();
	    for (int i = 0; i < stringBytes.length; i++) bytesToDisplay[i + 1] = stringBytes[i];
	    DatagramPacket packet = new DatagramPacket(bytesToDisplay,
		bytesToDisplay.length,
		InetAddress.getLocalHost(),
		Pad.PORT);
	    DatagramSocket socket = new DatagramSocket();
	    socket.send(packet);
	    socket.close();
	} catch(Exception e) {
	    if(debug) System.out.println("Error while sending message to StateTree!");
	}
    }

    protected void write2Tree(int ID, byte art, int wichtigkeit, String name, String string) {
	try {

	    if(name==null) name="Test";
	    String treeString = applicationName + ": " + string;

	    String treeName   = name;
	    byte[] bytesToDisplay = new byte[treeString.length()+ name.length() + 4];

	    bytesToDisplay[0] = (byte)ID;
	    bytesToDisplay[1] = (byte)art;
	    bytesToDisplay[2] = (byte)wichtigkeit;

	    bytesToDisplay[3] = (byte)name.length();

	    byte[] stringBytesName = treeName.getBytes();
	    byte[] stringBytesString = treeString.getBytes();
	    for (int i = 0; i < treeName.length(); i++)

		bytesToDisplay[i + 4] = stringBytesName[i];
	    for (int i = treeName.length(); i < (treeName.length()+treeString.length()); i++)
		bytesToDisplay[i + 4] = stringBytesString[i-treeName.length()];


	    DatagramPacket packet = new DatagramPacket(bytesToDisplay,
		bytesToDisplay.length,
		InetAddress.getLocalHost(),
		StateTree.PORT);
	    DatagramSocket socket = new DatagramSocket();
	    socket.send(packet);
	    socket.close();
	} catch(Exception e) {
	    if(debug) {
		System.out.println("Error while sending message to StateTree!");
		e.printStackTrace();
	    }
	}
    }
}
